/**
 * Represents one dialogue game between two players an opponent and a proponent about the 
 * classification of one single case
 *
 *
 */

package paduainterface;

public class DialogueGame{
  
  /** players*/
  private Player pro; 
  private Player opp;
  /**the game active history*/
  private ActiveHistory gameHistory = null;
  private int rounds;
  /**Constructor*/
  public DialogueGame (short proGoal, short oppGoal, double support) {
      this.pro = new Player(proGoal,oppGoal, support);
      this.opp = new Player(oppGoal,proGoal, support*5);
      this.rounds=0;
  }
  public DialogueGame (short proGoal, short oppGoal, double[] support) {
      this.pro = new Player(proGoal,oppGoal, support[0]);
      this.opp = new Player(oppGoal,proGoal, support[1]);
      this.rounds=0;
  }  
  
  /**setters and getters*/
  protected Player getProponent(){return this.pro;}
  protected Player getOpponent(){return this.opp;}
  protected ActiveHistory getGameHistory(){return this.gameHistory;}
  protected int getRounds(){return this.rounds;}
  /**set strategy*/
  protected void setStrategy(Strategy proStrategy, Strategy oppStrategy){
     this.pro.setStrategy(proStrategy);
     this.opp.setStrategy(oppStrategy);
  }
  /** get data*/
  protected void getData(String proDataFile, String oppDataFile){
    this.pro.getData(proDataFile);
    this.opp.getData(oppDataFile);    
    //build TTrees
    this.pro.buildTTree();
    this.opp.buildTTree();
  }
 
  /**set the current case*/
  private void setCase(short[] inputCase){
      pro.getCase(inputCase);
      opp.getCase(inputCase);
  }
 
  /**play one game and returns the classification result*/
  public short playGame(short[] currentCase){
    //dialogue
    this.setCase(currentCase);
    this.gameHistory = new ActiveHistory();

    Move firstMove = pro.playFirstMove(this.gameHistory);
    Move move1, move2;
    boolean opponentPlayedFirst=false;
    if(firstMove==null){
        firstMove=opp.playFirstMove(this.gameHistory);
        opponentPlayedFirst=true;
    }
    if (firstMove != null){
        if(!opponentPlayedFirst)
            this.gameHistory.insertRuleintoActiveHistory(pro.getGoalClass(), firstMove);
        else
            this.gameHistory.insertRuleintoActiveHistory(opp.getGoalClass(), firstMove);
        this.rounds++;
        Move lastMove = firstMove;
        while(lastMove != null){
            this.rounds++;
            if(!opponentPlayedFirst)
                move1 = opp.play(lastMove, this.gameHistory);
            else
                move1 = pro.play(lastMove, this.gameHistory);
            if (move1 != null){
                this.rounds++;
                if(!opponentPlayedFirst)
                move2 = pro.play(move1, this.gameHistory);
                else
                move2 = opp.play(move1, this.gameHistory);    
                lastMove = move2;
            }
            else
                lastMove = move1;
        }
    }
    short actualWinner = this.gameHistory.getWinner();
    return actualWinner;    
  }
}//end of the class
