/*
 * GameDictionary.java
 *
 * Created on 09 September 2008, 15:06
 *
 * Load Game Dictionay File and translate its components
 * Game Dictionary are layed out as following:
 * 0) the classes number
 * 1) the attributes number
 * 2) the word ATTRIBUTE
 * 3) the the attribute name
 * 4) the word VALUE
 * 5) the attribute values: string/string/string
 * @author maya
 */

package paduainterface;
//java Packages
import java.io.*;
import java.util.StringTokenizer;

public class GameDictionary {
    private File gameDictionaryPath = null;
    private BufferedReader gameDictionaryReader=null;
    private String [][]  attributes=null;//[name][value][value][value]
    private int classesNumber =0;

    /** Creates a new instance of GameDictionary */
    public GameDictionary(File path) {
        this.gameDictionaryPath = path;
    }

    //SETTERS and GETTERs
    protected int getClassesNumber(){return this.classesNumber;}
    protected int getAtrributesNumber(){
        if(this.attributes==null) return 0;
        else return this.attributes.length;
    }
    protected String[] getNonClassAttributes(){
        if(this.attributes==null) return null;
        if(this.attributes.length==1) return null;
        String[] attributesNames = new String[this.attributes.length-1];
        for(int i=0; i<this.attributes.length-1; i++)
            attributesNames[i] = this.attributes[i][0];
        return attributesNames;
    }
    protected String[] getAttributes(int index){
        if(this.attributes==null) return null;
        if((index>this.attributes.length)&&(index<0)) return null;
        String[] values = new String[attributes[index].length-1];
        for(int i=1; i<this.attributes[index].length; i++){
            values[i-1] = this.attributes[index][i];
        }
        return values;
    }
    protected String[] getClasses(){
        if(this.attributes==null) return null;
        int classIndex = this.attributes.length-1;
        String[] classes = new String[attributes[classIndex].length-1];
        for(int i=1; i<this.attributes[classIndex].length; i++){
            classes[i-1] = this.attributes[classIndex][i];
        }
        return classes;
    }

    protected short[] getClassesAttributes(){
        if (this.attributes==null) return null;
        String[] classes = this.getClasses();
        short[] classesAttributes = new short[classes.length];
        for(int i=0; i<classesAttributes.length; i++){
            classesAttributes[i] = this.link(classes[i],"CLASS");
        }
        return classesAttributes;
    }

    /**Opens the game dictionary file*/
    private boolean openGameDictionary(){
        try{
            /*
	    fileInput = new BufferedReader(file);*/
            FileReader file = new FileReader(this.gameDictionaryPath);
            this.gameDictionaryReader = new BufferedReader(file);
            return true;
        }
        catch(FileNotFoundException e){
            return false;

        }
    }
    /**Closes the game dictionary file*/
    private boolean closeGameDictionary(){
        try{
            this.gameDictionaryReader.close();
            return true;
        }
        catch(IOException e){
            return false;

        }
    }

    /**Translates the game dictionary file and returns:
       * 0 if the procedure was performed successuflly
       * -1 if failed to open/closing the file
       * -2 if failed to read the file
       * -3 if the file is not in the right formate*/
    protected int translate(){
        if (!this.openGameDictionary()) return -1;
        try{
            //get the classes Number
            String line = this.gameDictionaryReader.readLine();
            if(line==null) return -3;
            try{this.classesNumber = Integer.parseInt(line);}
            catch(NumberFormatException n){return -3;}
            //get the attributes Number
            line = this.gameDictionaryReader.readLine();
            if(line==null) return -3;
            int attributesNumber;
            try{attributesNumber = Integer.parseInt(line);}
            catch(NumberFormatException n){return -3;}
            //parse blocks
            String[][] tempAttributes  = new String[attributesNumber][];
            int index=0;

            line = this.gameDictionaryReader.readLine();
            if(!line.equalsIgnoreCase("ATTRIBUTE")) return -3;
            String[] parsedBlock = this.parseBlock();
            if(parsedBlock==null) return -3;
            while(parsedBlock!=null){
                tempAttributes[index] = new String[parsedBlock.length];
                for(int i=0; i<parsedBlock.length; i++)
                    tempAttributes[index][i] = parsedBlock[i];
                index++;
                parsedBlock=this.parseBlock();
            }
            if(index==attributesNumber) {
                this.attributes = tempAttributes;
                return 0;
            }
            else return -3;
        }
        catch(IOException e){
            return -2;
        }
    }

    private String[] parseBlock(){
        try{
           String[] parsedLine=null;
           String line = this.gameDictionaryReader.readLine();
           if(line==null) return null;
           if ((line.equalsIgnoreCase("ATTRIBUTE"))||(line.equalsIgnoreCase("VALUES")))
               return null;
           parsedLine = this.append(parsedLine, line);
           line = this.gameDictionaryReader.readLine();
           if(!line.equalsIgnoreCase("VALUES")) return null;
           line = this.gameDictionaryReader.readLine();
           if(line.equalsIgnoreCase("ATTRIBUTE")) return null;
           while((line!=null)&&(!line.equalsIgnoreCase("ATTRIBUTE"))){
               String [] lineAttributes = this.parseNamesLine(line);
               parsedLine = this.append(parsedLine, lineAttributes);
               line = this.gameDictionaryReader.readLine();
           }
           return parsedLine;
        }
        catch(IOException e){
            return null;
        }
    }
    private String[] parseNamesLine(String line){
        StringTokenizer tokenizer = new StringTokenizer(line,"//");
        int tokensNum = tokenizer.countTokens();
        if(tokensNum==0) return null;
        String[] namesInLine = new String[tokensNum];
        for(int i=0; i<tokensNum; i++){
           String token = tokenizer.nextToken();
           namesInLine[i]= token;
        }
        return namesInLine;
    }

    /**links attribute name to its short value*/
    protected short[] linkCase(String[] stringCase){
        if (this.attributes==null) return null;
        short attributeCounter=1;
        short[] newCase = new short[stringCase.length];
        for(int i=0; i<stringCase.length; i++){
            for(int j=1; j<this.attributes[i].length; j++){
                if(attributes[i][j].equalsIgnoreCase(stringCase[i])){
                    newCase[i] = attributeCounter;
                }
                attributeCounter++;
            }
        }
        return newCase;
    }
    protected short link(String attributeName, String attributeType){
        if (this.attributes==null) return -1;
        short attribute=1;
        for(int i=0; i <this.attributes.length; i++){
            if(this.attributes[i][0].equalsIgnoreCase(attributeType)){
                for(int j=1; j<this.attributes[i].length; j++){
                    if(this.attributes[i][j].equalsIgnoreCase(attributeName)){
                        return attribute;
                    }
                    else attribute++;
                }
            }
            else
              for(int j=1; j<this.attributes[i].length; j++)
                  attribute++;
        }
        return -1;
    }
    /**links attribute short value to its name*/
    protected String link(short attribute){
        if(this.attributes==null) return null;
        if(attribute<0) return null;
        short attributeNo=1;
        for(int i=0; i<this.attributes.length; i++){
            for(int j=1; j<this.attributes[i].length; j++){
                if(attributeNo==attribute) return this.attributes[i][j];
                else attributeNo++;
            }
        }
        return null;
    }

    protected String linkOutput(short attribute){
        if(this.attributes==null) return null;
        if(attribute<0) return null;
        short attributeNo=1;
        for(int i=0; i<this.attributes.length; i++){
            for(int j=1; j<this.attributes[i].length; j++){
                if(attributeNo==attribute) 
                    return this.attributes[i][0] + " = " + this.attributes[i][j];
                else attributeNo++;
            }
        }
        return null;
    }
    /**returns the number of attributes in the attributes array (including classes)*/
    protected short attributesNumber(){
        if (this.attributes==null) return 0;
        short attributesNo = 0;
        for(int i=0; i<this.attributes.length; i++){
            for(int j=1; j<this.attributes[i].length; j++)
                attributesNo++;
        }
        return attributesNo;
    }
    /**outputs the dictionary information in a string format*/
    protected String output(){
        if(this.attributes==null) return null;
        short attributeShort=1;
        String output = "Game Dictionary :\n";
        for(int i=0; i<this.attributes.length; i++){
            output+= this.attributes[i][0]+" :\n";
            for(int j=1; j<this.attributes[i].length-1; j++){
                output+= this.attributes[i][j]+" : " + attributeShort + " - ";
                attributeShort++;
            }
            output+= this.attributes[i][this.attributes[i].length-1]+" : "
                    + attributeShort + "\n ";
            attributeShort++;
        }
        return output;
    }

    private String[] copy(String[] stringSet) {
	if (stringSet == null) return(null);
	String[] newStringSet = new String[stringSet.length];
	for(int index=0;index<stringSet.length;index++) {
	    newStringSet[index] = stringSet[index];
        }
	return(newStringSet);
    }
    private String[] copy(String item) {
	if (item==null) return null;
        String[] newStringSet = new String[1];
        newStringSet[0] = item;
	return(newStringSet);
    }
    private String[] append(String[] itemSet1, String[] itemSet2) {
	if (itemSet1 == null) return(copy(itemSet2));
	else if (itemSet2 == null) return(copy(itemSet1));
	String[] newItemSet = new String[itemSet1.length+itemSet2.length];
	int index1;
	for(index1=0;index1<itemSet1.length;index1++) {
	    newItemSet[index1]=itemSet1[index1];
	    }
	for(int index2=0;index2<itemSet2.length;index2++) {
	    newItemSet[index1+index2]=itemSet2[index2];
	    }
	return(newItemSet);
        }
    private String[] append(String[] itemSet, String item) {
	if (itemSet == null) return(copy(item));
	else if (item == null) return(copy(itemSet));
	String[] newItemSet = new String[itemSet.length+1];
	for(int index=0;index<itemSet.length;index++) {
	    newItemSet[index]=itemSet[index];
	    }
        newItemSet[itemSet.length] = item;
	return(newItemSet);
        }
}

