/*
 * PADUAInterfaceView.java
 */

package paduainterface;

import org.jdesktop.application.Action;
import org.jdesktop.application.ResourceMap;
import org.jdesktop.application.SingleFrameApplication;
import org.jdesktop.application.FrameView;
import org.jdesktop.application.TaskMonitor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.Timer;
import javax.swing.Icon;
import javax.swing.JDialog;
import javax.swing.JFrame;
import java.io.File;
import java.awt.Color;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
/**
 * The application's main frame.
 */
public class PADUAInterfaceView extends FrameView {
    private GameDictionary gameDictionary=null;
    private short[] currentCase=null;
    private String proFile;
    private String oppFile;
    private DialogueGame dialogueGame;
    private DynamicCaseDialog customDialog;
    private Strategy proStrategy=new Strategy();
    private Strategy oppStrategy=new Strategy();
    public PADUAInterfaceView(SingleFrameApplication app) {
        super(app);
        this.getFrame().setTitle("PADUA GUI Application");
        initComponents();
        // status bar initialization - message timeout, idle icon and busy animation, etc
        ResourceMap resourceMap = getResourceMap();
        int messageTimeout = resourceMap.getInteger("StatusBar.messageTimeout");
        messageTimer = new Timer(messageTimeout, new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                statusMessageLabel.setText("");
            }
        });
        messageTimer.setRepeats(false);
        int busyAnimationRate = resourceMap.getInteger("StatusBar.busyAnimationRate");
        for (int i = 0; i < busyIcons.length; i++) {
            busyIcons[i] = resourceMap.getIcon("StatusBar.busyIcons[" + i + "]");
        }
        busyIconTimer = new Timer(busyAnimationRate, new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                busyIconIndex = (busyIconIndex + 1) % busyIcons.length;
                statusAnimationLabel.setIcon(busyIcons[busyIconIndex]);
            }
        });
        idleIcon = resourceMap.getIcon("StatusBar.idleIcon");
        statusAnimationLabel.setIcon(idleIcon);
        progressBar.setVisible(false);

        // connecting action tasks to status bar via TaskMonitor
        TaskMonitor taskMonitor = new TaskMonitor(getApplication().getContext());
        taskMonitor.addPropertyChangeListener(new java.beans.PropertyChangeListener() {
            public void propertyChange(java.beans.PropertyChangeEvent evt) {
                String propertyName = evt.getPropertyName();
                if ("started".equals(propertyName)) {
                    if (!busyIconTimer.isRunning()) {
                        statusAnimationLabel.setIcon(busyIcons[0]);
                        busyIconIndex = 0;
                        busyIconTimer.start();
                    }
                    progressBar.setVisible(true);
                    progressBar.setIndeterminate(true);
                } else if ("done".equals(propertyName)) {
                    busyIconTimer.stop();
                    statusAnimationLabel.setIcon(idleIcon);
                    progressBar.setVisible(false);
                    progressBar.setValue(0);
                } else if ("message".equals(propertyName)) {
                    String text = (String)(evt.getNewValue());
                    statusMessageLabel.setText((text == null) ? "" : text);
                    messageTimer.restart();
                } else if ("progress".equals(propertyName)) {
                    int value = (Integer)(evt.getNewValue());
                    progressBar.setVisible(true);
                    progressBar.setIndeterminate(false);
                    progressBar.setValue(value);
                }
            }
        });
    }

    @Action
    public void showAboutBox() {
        if (aboutBox == null) {
            JFrame mainFrame = PADUAInterfaceApp.getApplication().getMainFrame();
            aboutBox = new PADUAInterfaceAboutBox(mainFrame);
            aboutBox.setLocationRelativeTo(mainFrame);
        }
        PADUAInterfaceApp.getApplication().show(aboutBox);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        mainPanel = new javax.swing.JPanel();
        mainTabbedPane = new javax.swing.JTabbedPane();
        generalInfoScrollPane = new javax.swing.JScrollPane();
        generalInfoTextArea = new javax.swing.JTextArea();
        jScrollPane1 = new javax.swing.JScrollPane();
        playersInfoTextArea = new javax.swing.JTextArea();
        dialogueGameScrollPane = new javax.swing.JScrollPane();
        dialogueGameTextArea = new javax.swing.JTextArea();
        menuBar = new javax.swing.JMenuBar();
        javax.swing.JMenu gameSettingsMenu = new javax.swing.JMenu();
        gameMenuItem = new javax.swing.JMenuItem();
        playersMenu = new javax.swing.JMenu();
        proMenuItem = new javax.swing.JMenuItem();
        oppMenuItem = new javax.swing.JMenuItem();
        javax.swing.JMenuItem exitMenuItem = new javax.swing.JMenuItem();
        playGameMenu = new javax.swing.JMenu();
        loadCaseMenuItem = new javax.swing.JMenuItem();
        strategyMenu = new javax.swing.JMenu();
        proStrategyMenuItem = new javax.swing.JMenuItem();
        oppStrategyMenuItem = new javax.swing.JMenuItem();
        playMenu = new javax.swing.JMenu();
        posGameMenuItem = new javax.swing.JMenuItem();
        negGameMenuItem = new javax.swing.JMenuItem();
        javax.swing.JMenu helpMenu = new javax.swing.JMenu();
        javax.swing.JMenuItem aboutMenuItem = new javax.swing.JMenuItem();
        statusPanel = new javax.swing.JPanel();
        javax.swing.JSeparator statusPanelSeparator = new javax.swing.JSeparator();
        statusMessageLabel = new javax.swing.JLabel();
        statusAnimationLabel = new javax.swing.JLabel();
        progressBar = new javax.swing.JProgressBar();
        proStrategyDialog = new javax.swing.JDialog();
        gameModejLabel = new javax.swing.JLabel();
        proStrategyDoneButton = new javax.swing.JButton();
        proGameModeComboBox = new javax.swing.JComboBox();
        agentProfilejLabel = new javax.swing.JLabel();
        proAgentProfileComboBox = new javax.swing.JComboBox();
        jSeparator3 = new javax.swing.JSeparator();
        armOptionsjLabel = new javax.swing.JLabel();
        minConfjLabel = new javax.swing.JLabel();
        maxConfjLabel = new javax.swing.JLabel();
        distThresholdjLabel = new javax.swing.JLabel();
        upThresholdjLabel = new javax.swing.JLabel();
        maxLeveljLabel = new javax.swing.JLabel();
        consideredLevelsjLabel = new javax.swing.JLabel();
        proMiinConfTextField = new javax.swing.JTextField();
        proDistThresholdTextField = new javax.swing.JTextField();
        proMaxLevelsTextField = new javax.swing.JTextField();
        proMaxConfTextField = new javax.swing.JTextField();
        proUpThresholdTextField = new javax.swing.JTextField();
        proConsideredLevelsTextField = new javax.swing.JTextField();
        strategyModeLabel = new javax.swing.JLabel();
        proStrategyModeComboBox = new javax.swing.JComboBox();
        oppStrategyDialog = new javax.swing.JDialog();
        gameModejLabel1 = new javax.swing.JLabel();
        oppStrategyOptionsDonejButton = new javax.swing.JButton();
        oppGameModeComboBox = new javax.swing.JComboBox();
        agentProfilejLabel1 = new javax.swing.JLabel();
        oppAgentProfileComboBox = new javax.swing.JComboBox();
        jSeparator5 = new javax.swing.JSeparator();
        armOptionsjLabel1 = new javax.swing.JLabel();
        minConfjLabel1 = new javax.swing.JLabel();
        maxConfjLabel1 = new javax.swing.JLabel();
        distThresholdjLabel1 = new javax.swing.JLabel();
        upThresholdjLabel1 = new javax.swing.JLabel();
        maxLeveljLabel1 = new javax.swing.JLabel();
        consideredLevelsjLabel1 = new javax.swing.JLabel();
        oppMinConfTextField = new javax.swing.JTextField();
        oppDistThresholdTextField = new javax.swing.JTextField();
        oppMaxLevelsTextField = new javax.swing.JTextField();
        oppMaxConfTextField = new javax.swing.JTextField();
        oppUpThresholdTextField = new javax.swing.JTextField();
        oppConsideredLevelsTextField = new javax.swing.JTextField();
        strategyModeLabel1 = new javax.swing.JLabel();
        oppStrategyModeComboBox = new javax.swing.JComboBox();
        helpDialoge = new javax.swing.JDialog();
        jLabel1 = new javax.swing.JLabel();
        helpCloseButton = new javax.swing.JButton();

        mainPanel.setName("mainPanel"); // NOI18N

        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(paduainterface.PADUAInterfaceApp.class).getContext().getResourceMap(PADUAInterfaceView.class);
        mainTabbedPane.setBackground(resourceMap.getColor("mainTabbedPane.background")); // NOI18N
        mainTabbedPane.setTabPlacement(javax.swing.JTabbedPane.LEFT);
        mainTabbedPane.setFont(resourceMap.getFont("mainTabbedPane.font")); // NOI18N
        mainTabbedPane.setName("mainTabbedPane"); // NOI18N
        mainTabbedPane.setPreferredSize(new java.awt.Dimension(420, 318));
        mainTabbedPane.setRequestFocusEnabled(false);
        mainTabbedPane.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                mainTabbedPaneStateChanged(evt);
            }
        });

        generalInfoScrollPane.setName("generalInfoScrollPane"); // NOI18N

        generalInfoTextArea.setBackground(resourceMap.getColor("generalInfoTextArea.background")); // NOI18N
        generalInfoTextArea.setColumns(20);
        generalInfoTextArea.setFont(resourceMap.getFont("generalInfoTextArea.font")); // NOI18N
        generalInfoTextArea.setRows(5);
        generalInfoTextArea.setText(resourceMap.getString("generalInfoTextArea.text")); // NOI18N
        generalInfoTextArea.setName("generalInfoTextArea"); // NOI18N
        generalInfoScrollPane.setViewportView(generalInfoTextArea);

        mainTabbedPane.addTab(resourceMap.getString("generalInfoScrollPane.TabConstraints.tabTitle"), generalInfoScrollPane); // NOI18N

        jScrollPane1.setName("jScrollPane1"); // NOI18N

        playersInfoTextArea.setBackground(resourceMap.getColor("playersInfoTextArea.background")); // NOI18N
        playersInfoTextArea.setColumns(20);
        playersInfoTextArea.setFont(resourceMap.getFont("playersInfoTextArea.font")); // NOI18N
        playersInfoTextArea.setRows(5);
        playersInfoTextArea.setText(resourceMap.getString("playersInfoTextArea.text")); // NOI18N
        playersInfoTextArea.setName("playersInfoTextArea"); // NOI18N
        jScrollPane1.setViewportView(playersInfoTextArea);

        mainTabbedPane.addTab(resourceMap.getString("jScrollPane1.TabConstraints.tabTitle"), jScrollPane1); // NOI18N

        dialogueGameScrollPane.setName("dialogueGameScrollPane"); // NOI18N

        dialogueGameTextArea.setBackground(resourceMap.getColor("dialogueGameTextArea.background")); // NOI18N
        dialogueGameTextArea.setColumns(20);
        dialogueGameTextArea.setFont(resourceMap.getFont("dialogueGameTextArea.font")); // NOI18N
        dialogueGameTextArea.setRows(5);
        dialogueGameTextArea.setText(resourceMap.getString("dialogueGameTextArea.text")); // NOI18N
        dialogueGameTextArea.setName("dialogueGameTextArea"); // NOI18N
        dialogueGameScrollPane.setViewportView(dialogueGameTextArea);

        mainTabbedPane.addTab(resourceMap.getString("dialogueGameScrollPane.TabConstraints.tabTitle"), dialogueGameScrollPane); // NOI18N

        javax.swing.GroupLayout mainPanelLayout = new javax.swing.GroupLayout(mainPanel);
        mainPanel.setLayout(mainPanelLayout);
        mainPanelLayout.setHorizontalGroup(
            mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(mainPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(mainTabbedPane, javax.swing.GroupLayout.DEFAULT_SIZE, 784, Short.MAX_VALUE)
                .addContainerGap())
        );
        mainPanelLayout.setVerticalGroup(
            mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(mainPanelLayout.createSequentialGroup()
                .addComponent(mainTabbedPane, javax.swing.GroupLayout.DEFAULT_SIZE, 458, Short.MAX_VALUE)
                .addContainerGap())
        );

        menuBar.setName("menuBar"); // NOI18N

        gameSettingsMenu.setText(resourceMap.getString("gameSettingsMenu.text")); // NOI18N
        gameSettingsMenu.setName("gameSettingsMenu"); // NOI18N

        gameMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_G, java.awt.event.InputEvent.CTRL_MASK));
        gameMenuItem.setText(resourceMap.getString("gameMenuItem.text")); // NOI18N
        gameMenuItem.setName("gameMenuItem"); // NOI18N
        gameMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                gameMenuItemActionPerformed(evt);
            }
        });
        gameSettingsMenu.add(gameMenuItem);

        playersMenu.setText(resourceMap.getString("playersMenu.text")); // NOI18N
        playersMenu.setName("playersMenu"); // NOI18N

        proMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_P, java.awt.event.InputEvent.CTRL_MASK));
        proMenuItem.setText(resourceMap.getString("proMenuItem.text")); // NOI18N
        proMenuItem.setName("proMenuItem"); // NOI18N
        proMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                proMenuItemActionPerformed(evt);
            }
        });
        playersMenu.add(proMenuItem);

        oppMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_O, java.awt.event.InputEvent.CTRL_MASK));
        oppMenuItem.setText(resourceMap.getString("oppMenuItem.text")); // NOI18N
        oppMenuItem.setName("oppMenuItem"); // NOI18N
        oppMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                oppMenuItemActionPerformed(evt);
            }
        });
        playersMenu.add(oppMenuItem);

        gameSettingsMenu.add(playersMenu);

        javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(paduainterface.PADUAInterfaceApp.class).getContext().getActionMap(PADUAInterfaceView.class, this);
        exitMenuItem.setAction(actionMap.get("quit")); // NOI18N
        exitMenuItem.setName("exitMenuItem"); // NOI18N
        gameSettingsMenu.add(exitMenuItem);

        menuBar.add(gameSettingsMenu);

        playGameMenu.setText(resourceMap.getString("playMenu.text")); // NOI18N
        playGameMenu.setEnabled(false);
        playGameMenu.setName("playMenu"); // NOI18N

        loadCaseMenuItem.setText(resourceMap.getString("loadCaseMenuItem.text")); // NOI18N
        loadCaseMenuItem.setName("loadCaseMenuItem"); // NOI18N
        loadCaseMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                loadCaseMenuItemActionPerformed(evt);
            }
        });
        playGameMenu.add(loadCaseMenuItem);

        strategyMenu.setText(resourceMap.getString("strategyMenu.text")); // NOI18N
        strategyMenu.setName("strategyMenu"); // NOI18N

        proStrategyMenuItem.setText(resourceMap.getString("proStrategyMenuItem.text")); // NOI18N
        proStrategyMenuItem.setName("proStrategyMenuItem"); // NOI18N
        proStrategyMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                proStrategyMenuItemActionPerformed(evt);
            }
        });
        strategyMenu.add(proStrategyMenuItem);

        oppStrategyMenuItem.setText(resourceMap.getString("oppStrategyMenuItem.text")); // NOI18N
        oppStrategyMenuItem.setName("oppStrategyMenuItem"); // NOI18N
        oppStrategyMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                oppStrategyMenuItemActionPerformed(evt);
            }
        });
        strategyMenu.add(oppStrategyMenuItem);

        playGameMenu.add(strategyMenu);

        playMenu.setText(resourceMap.getString("playMenu.text")); // NOI18N
        playMenu.setName("playMenu"); // NOI18N

        posGameMenuItem.setText(resourceMap.getString("posGameMenuItem.text")); // NOI18N
        posGameMenuItem.setName("posGameMenuItem"); // NOI18N
        posGameMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                posGameMenuItemActionPerformed(evt);
            }
        });
        playMenu.add(posGameMenuItem);

        negGameMenuItem.setText(resourceMap.getString("negGameMenuItem.text")); // NOI18N
        negGameMenuItem.setName("negGameMenuItem"); // NOI18N
        negGameMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                negGameMenuItemActionPerformed(evt);
            }
        });
        playMenu.add(negGameMenuItem);

        playGameMenu.add(playMenu);

        menuBar.add(playGameMenu);

        helpMenu.setText(resourceMap.getString("helpMenu.text")); // NOI18N
        helpMenu.setName("helpMenu"); // NOI18N

        aboutMenuItem.setAction(actionMap.get("showAboutBox")); // NOI18N
        aboutMenuItem.setName("aboutMenuItem"); // NOI18N
        aboutMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                aboutMenuItemActionPerformed(evt);
            }
        });
        helpMenu.add(aboutMenuItem);

        menuBar.add(helpMenu);

        statusPanel.setName("statusPanel"); // NOI18N

        statusPanelSeparator.setName("statusPanelSeparator"); // NOI18N

        statusMessageLabel.setName("statusMessageLabel"); // NOI18N

        statusAnimationLabel.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        statusAnimationLabel.setName("statusAnimationLabel"); // NOI18N

        progressBar.setName("progressBar"); // NOI18N

        javax.swing.GroupLayout statusPanelLayout = new javax.swing.GroupLayout(statusPanel);
        statusPanel.setLayout(statusPanelLayout);
        statusPanelLayout.setHorizontalGroup(
            statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(statusPanelSeparator, javax.swing.GroupLayout.DEFAULT_SIZE, 804, Short.MAX_VALUE)
            .addGroup(statusPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(statusMessageLabel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 634, Short.MAX_VALUE)
                .addComponent(progressBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(statusAnimationLabel)
                .addContainerGap())
        );
        statusPanelLayout.setVerticalGroup(
            statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(statusPanelLayout.createSequentialGroup()
                .addComponent(statusPanelSeparator, javax.swing.GroupLayout.PREFERRED_SIZE, 2, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(statusMessageLabel)
                    .addComponent(statusAnimationLabel)
                    .addComponent(progressBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(3, 3, 3))
        );

        proStrategyDialog.setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        proStrategyDialog.setTitle(resourceMap.getString("proStrategyDialog.title")); // NOI18N
        proStrategyDialog.setName("proStrategyDialog"); // NOI18N

        gameModejLabel.setText(resourceMap.getString("gameModejLabel.text")); // NOI18N
        gameModejLabel.setName("gameModejLabel"); // NOI18N

        proStrategyDoneButton.setText(resourceMap.getString("proStrategyDoneButton.text")); // NOI18N
        proStrategyDoneButton.setName("proStrategyDoneButton"); // NOI18N
        proStrategyDoneButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                proStrategyDoneButtonActionPerformed(evt);
            }
        });

        proGameModeComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Game", "Win" }));
        proGameModeComboBox.setName("proGameModeComboBox"); // NOI18N

        agentProfilejLabel.setText(resourceMap.getString("agentProfilejLabel.text")); // NOI18N
        agentProfilejLabel.setName("agentProfilejLabel"); // NOI18N

        proAgentProfileComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Agreeable", "Disagreeable" }));
        proAgentProfileComboBox.setName("proAgentProfileComboBox"); // NOI18N

        jSeparator3.setName("jSeparator3"); // NOI18N

        armOptionsjLabel.setText(resourceMap.getString("armOptionsjLabel.text")); // NOI18N
        armOptionsjLabel.setName("armOptionsjLabel"); // NOI18N

        minConfjLabel.setText(resourceMap.getString("minConfjLabel.text")); // NOI18N
        minConfjLabel.setName("minConfjLabel"); // NOI18N

        maxConfjLabel.setText(resourceMap.getString("maxConfjLabel.text")); // NOI18N
        maxConfjLabel.setName("maxConfjLabel"); // NOI18N

        distThresholdjLabel.setText(resourceMap.getString("distThresholdjLabel.text")); // NOI18N
        distThresholdjLabel.setName("distThresholdjLabel"); // NOI18N

        upThresholdjLabel.setText(resourceMap.getString("upThresholdjLabel.text")); // NOI18N
        upThresholdjLabel.setName("upThresholdjLabel"); // NOI18N

        maxLeveljLabel.setText(resourceMap.getString("maxLeveljLabel.text")); // NOI18N
        maxLeveljLabel.setName("maxLeveljLabel"); // NOI18N

        consideredLevelsjLabel.setText(resourceMap.getString("consideredLevelsjLabel.text")); // NOI18N
        consideredLevelsjLabel.setName("consideredLevelsjLabel"); // NOI18N

        proMiinConfTextField.setText(resourceMap.getString("proMiinConfTextField.text")); // NOI18N
        proMiinConfTextField.setName("proMiinConfTextField"); // NOI18N

        proDistThresholdTextField.setText(resourceMap.getString("proDistThresholdTextField.text")); // NOI18N
        proDistThresholdTextField.setName("proDistThresholdTextField"); // NOI18N

        proMaxLevelsTextField.setText(resourceMap.getString("proMaxLevelsTextField.text")); // NOI18N
        proMaxLevelsTextField.setName("proMaxLevelsTextField"); // NOI18N

        proMaxConfTextField.setText(resourceMap.getString("proMaxConfTextField.text")); // NOI18N
        proMaxConfTextField.setName("proMaxConfTextField"); // NOI18N

        proUpThresholdTextField.setText(resourceMap.getString("proUpThresholdTextField.text")); // NOI18N
        proUpThresholdTextField.setName("proUpThresholdTextField"); // NOI18N

        proConsideredLevelsTextField.setText(resourceMap.getString("proConsideredLevelsTextField.text")); // NOI18N
        proConsideredLevelsTextField.setName("proConsideredLevelsTextField"); // NOI18N

        strategyModeLabel.setText(resourceMap.getString("strategyModeLabel.text")); // NOI18N
        strategyModeLabel.setName("strategyModeLabel"); // NOI18N

        proStrategyModeComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Build", "Destroy" }));
        proStrategyModeComboBox.setName("proStrategyModeComboBox"); // NOI18N

        javax.swing.GroupLayout proStrategyDialogLayout = new javax.swing.GroupLayout(proStrategyDialog.getContentPane());
        proStrategyDialog.getContentPane().setLayout(proStrategyDialogLayout);
        proStrategyDialogLayout.setHorizontalGroup(
            proStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(proStrategyDialogLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(strategyModeLabel)
                .addGap(12, 12, 12)
                .addComponent(proStrategyModeComboBox, 0, 125, Short.MAX_VALUE)
                .addGap(223, 223, 223))
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, proStrategyDialogLayout.createSequentialGroup()
                .addGroup(proStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, proStrategyDialogLayout.createSequentialGroup()
                        .addContainerGap()
                        .addGroup(proStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(agentProfilejLabel)
                            .addComponent(gameModejLabel))
                        .addGap(12, 12, 12)
                        .addGroup(proStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(proAgentProfileComboBox, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(proGameModeComboBox, 0, 126, Short.MAX_VALUE)))
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, proStrategyDialogLayout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(armOptionsjLabel)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 339, Short.MAX_VALUE))
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, proStrategyDialogLayout.createSequentialGroup()
                        .addContainerGap()
                        .addGroup(proStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(minConfjLabel)
                            .addComponent(distThresholdjLabel)
                            .addComponent(maxLeveljLabel))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(proStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(proMiinConfTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 81, Short.MAX_VALUE)
                            .addComponent(proMaxLevelsTextField)
                            .addComponent(proDistThresholdTextField))
                        .addGap(28, 28, 28)
                        .addGroup(proStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(maxConfjLabel)
                            .addComponent(upThresholdjLabel)
                            .addComponent(consideredLevelsjLabel))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 14, Short.MAX_VALUE)
                        .addGroup(proStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(proMaxConfTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 72, Short.MAX_VALUE)
                            .addComponent(proUpThresholdTextField)
                            .addComponent(proConsideredLevelsTextField)
                            .addComponent(proStrategyDoneButton, javax.swing.GroupLayout.Alignment.TRAILING))
                        .addGap(30, 30, 30)))
                .addContainerGap())
            .addComponent(jSeparator3, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 441, Short.MAX_VALUE)
        );
        proStrategyDialogLayout.setVerticalGroup(
            proStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(proStrategyDialogLayout.createSequentialGroup()
                .addGroup(proStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(proStrategyDialogLayout.createSequentialGroup()
                        .addContainerGap()
                        .addGroup(proStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(gameModejLabel)
                            .addComponent(proGameModeComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(proStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(agentProfilejLabel)
                            .addComponent(proAgentProfileComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(proStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(strategyModeLabel)
                            .addComponent(proStrategyModeComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addGap(16, 16, 16)
                        .addGroup(proStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(proStrategyDialogLayout.createSequentialGroup()
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(armOptionsjLabel)
                                .addGap(15, 15, 15)
                                .addGroup(proStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                    .addComponent(minConfjLabel)
                                    .addComponent(proMiinConfTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                                .addGap(22, 22, 22)
                                .addGroup(proStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                    .addComponent(distThresholdjLabel)
                                    .addComponent(proDistThresholdTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                                .addGap(26, 26, 26)
                                .addGroup(proStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                    .addComponent(maxLeveljLabel)
                                    .addComponent(proMaxLevelsTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                            .addGroup(proStrategyDialogLayout.createSequentialGroup()
                                .addGap(34, 34, 34)
                                .addGroup(proStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                    .addComponent(maxConfjLabel)
                                    .addComponent(proMaxConfTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                                .addGap(22, 22, 22)
                                .addGroup(proStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                    .addComponent(upThresholdjLabel)
                                    .addComponent(proUpThresholdTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                                .addGap(26, 26, 26)
                                .addGroup(proStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                    .addComponent(consideredLevelsjLabel)
                                    .addComponent(proConsideredLevelsTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))))
                        .addGap(18, 18, 18)
                        .addComponent(proStrategyDoneButton))
                    .addGroup(proStrategyDialogLayout.createSequentialGroup()
                        .addGap(94, 94, 94)
                        .addComponent(jSeparator3, javax.swing.GroupLayout.PREFERRED_SIZE, 10, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap(12, Short.MAX_VALUE))
        );

        oppStrategyDialog.setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        oppStrategyDialog.setTitle(resourceMap.getString("oppStrategyDialog.title")); // NOI18N
        oppStrategyDialog.setName("oppStrategyDialog"); // NOI18N

        gameModejLabel1.setText(resourceMap.getString("gameModejLabel1.text")); // NOI18N
        gameModejLabel1.setName("gameModejLabel1"); // NOI18N

        oppStrategyOptionsDonejButton.setText(resourceMap.getString("oppStrategyOptionsDonejButton.text")); // NOI18N
        oppStrategyOptionsDonejButton.setName("oppStrategyOptionsDonejButton"); // NOI18N
        oppStrategyOptionsDonejButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                oppStrategyOptionsDonejButtonActionPerformed(evt);
            }
        });

        oppGameModeComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Game", "Win" }));
        oppGameModeComboBox.setName("oppGameModeComboBox"); // NOI18N

        agentProfilejLabel1.setText(resourceMap.getString("agentProfilejLabel1.text")); // NOI18N
        agentProfilejLabel1.setName("agentProfilejLabel1"); // NOI18N

        oppAgentProfileComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Agreeable", "Disagreeable" }));
        oppAgentProfileComboBox.setName("oppAgentProfileComboBox"); // NOI18N

        jSeparator5.setName("jSeparator5"); // NOI18N

        armOptionsjLabel1.setText(resourceMap.getString("armOptionsjLabel1.text")); // NOI18N
        armOptionsjLabel1.setName("armOptionsjLabel1"); // NOI18N

        minConfjLabel1.setText(resourceMap.getString("minConfjLabel1.text")); // NOI18N
        minConfjLabel1.setName("minConfjLabel1"); // NOI18N

        maxConfjLabel1.setText(resourceMap.getString("maxConfjLabel1.text")); // NOI18N
        maxConfjLabel1.setName("maxConfjLabel1"); // NOI18N

        distThresholdjLabel1.setText(resourceMap.getString("distThresholdjLabel1.text")); // NOI18N
        distThresholdjLabel1.setName("distThresholdjLabel1"); // NOI18N

        upThresholdjLabel1.setText(resourceMap.getString("upThresholdjLabel1.text")); // NOI18N
        upThresholdjLabel1.setName("upThresholdjLabel1"); // NOI18N

        maxLeveljLabel1.setText(resourceMap.getString("maxLeveljLabel1.text")); // NOI18N
        maxLeveljLabel1.setName("maxLeveljLabel1"); // NOI18N

        consideredLevelsjLabel1.setText(resourceMap.getString("consideredLevelsjLabel1.text")); // NOI18N
        consideredLevelsjLabel1.setName("consideredLevelsjLabel1"); // NOI18N

        oppMinConfTextField.setText(resourceMap.getString("oppMinConfTextField.text")); // NOI18N
        oppMinConfTextField.setName("oppMinConfTextField"); // NOI18N

        oppDistThresholdTextField.setText(resourceMap.getString("oppDistThresholdTextField.text")); // NOI18N
        oppDistThresholdTextField.setName("oppDistThresholdTextField"); // NOI18N

        oppMaxLevelsTextField.setText(resourceMap.getString("oppMaxLevelsTextField.text")); // NOI18N
        oppMaxLevelsTextField.setName("oppMaxLevelsTextField"); // NOI18N

        oppMaxConfTextField.setText(resourceMap.getString("oppMaxConfTextField.text")); // NOI18N
        oppMaxConfTextField.setName("oppMaxConfTextField"); // NOI18N

        oppUpThresholdTextField.setText(resourceMap.getString("oppUpThresholdTextField.text")); // NOI18N
        oppUpThresholdTextField.setName("oppUpThresholdTextField"); // NOI18N

        oppConsideredLevelsTextField.setText(resourceMap.getString("oppConsideredLevelsTextField.text")); // NOI18N
        oppConsideredLevelsTextField.setName("oppConsideredLevelsTextField"); // NOI18N

        strategyModeLabel1.setText(resourceMap.getString("strategyModeLabel1.text")); // NOI18N
        strategyModeLabel1.setName("strategyModeLabel1"); // NOI18N

        oppStrategyModeComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Build", "Destroy" }));
        oppStrategyModeComboBox.setName("oppStrategyModeComboBox"); // NOI18N

        javax.swing.GroupLayout oppStrategyDialogLayout = new javax.swing.GroupLayout(oppStrategyDialog.getContentPane());
        oppStrategyDialog.getContentPane().setLayout(oppStrategyDialogLayout);
        oppStrategyDialogLayout.setHorizontalGroup(
            oppStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(oppStrategyDialogLayout.createSequentialGroup()
                .addGroup(oppStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(oppStrategyDialogLayout.createSequentialGroup()
                        .addContainerGap()
                        .addGroup(oppStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(agentProfilejLabel1)
                            .addComponent(gameModejLabel1))
                        .addGap(12, 12, 12)
                        .addGroup(oppStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(oppAgentProfileComboBox, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(oppGameModeComboBox, 0, 126, Short.MAX_VALUE)))
                    .addComponent(jSeparator5, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 452, Short.MAX_VALUE)
                    .addGroup(oppStrategyDialogLayout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(armOptionsjLabel1)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 380, Short.MAX_VALUE))
                    .addGroup(oppStrategyDialogLayout.createSequentialGroup()
                        .addContainerGap()
                        .addGroup(oppStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(minConfjLabel1)
                            .addComponent(distThresholdjLabel1)
                            .addComponent(maxLeveljLabel1))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(oppStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(oppMinConfTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 81, Short.MAX_VALUE)
                            .addComponent(oppMaxLevelsTextField)
                            .addComponent(oppDistThresholdTextField))
                        .addGap(28, 28, 28)
                        .addGroup(oppStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(maxConfjLabel1)
                            .addComponent(upThresholdjLabel1)
                            .addComponent(consideredLevelsjLabel1))
                        .addGap(14, 14, 14)
                        .addGroup(oppStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(oppMaxConfTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 72, Short.MAX_VALUE)
                            .addComponent(oppUpThresholdTextField)
                            .addComponent(oppConsideredLevelsTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 82, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(oppStrategyOptionsDonejButton, javax.swing.GroupLayout.Alignment.TRAILING))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 31, Short.MAX_VALUE)))
                .addContainerGap())
            .addGroup(oppStrategyDialogLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(strategyModeLabel1)
                .addGap(12, 12, 12)
                .addComponent(oppStrategyModeComboBox, 0, 125, Short.MAX_VALUE)
                .addGap(234, 234, 234))
        );
        oppStrategyDialogLayout.setVerticalGroup(
            oppStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(oppStrategyDialogLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(oppStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(gameModejLabel1)
                    .addComponent(oppGameModeComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(oppStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(agentProfilejLabel1)
                    .addComponent(oppAgentProfileComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(oppStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(strategyModeLabel1)
                    .addComponent(oppStrategyModeComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jSeparator5, javax.swing.GroupLayout.PREFERRED_SIZE, 10, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGroup(oppStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(oppStrategyDialogLayout.createSequentialGroup()
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(armOptionsjLabel1)
                        .addGap(15, 15, 15)
                        .addGroup(oppStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(minConfjLabel1)
                            .addComponent(oppMinConfTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addGap(22, 22, 22)
                        .addGroup(oppStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(distThresholdjLabel1)
                            .addComponent(oppDistThresholdTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addGap(26, 26, 26)
                        .addGroup(oppStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(maxLeveljLabel1)
                            .addComponent(oppMaxLevelsTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                    .addGroup(oppStrategyDialogLayout.createSequentialGroup()
                        .addGap(34, 34, 34)
                        .addGroup(oppStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(maxConfjLabel1)
                            .addComponent(oppMaxConfTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addGap(22, 22, 22)
                        .addGroup(oppStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(upThresholdjLabel1)
                            .addComponent(oppUpThresholdTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addGap(26, 26, 26)
                        .addGroup(oppStrategyDialogLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(consideredLevelsjLabel1)
                            .addComponent(oppConsideredLevelsTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))))
                .addGap(18, 18, 18)
                .addComponent(oppStrategyOptionsDonejButton)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        helpDialoge.setTitle(resourceMap.getString("helpDialoge.title")); // NOI18N
        helpDialoge.setName("helpDialoge"); // NOI18N

        jLabel1.setText(resourceMap.getString("jLabel1.text")); // NOI18N
        jLabel1.setName("jLabel1"); // NOI18N

        helpCloseButton.setText(resourceMap.getString("helpCloseButton.text")); // NOI18N
        helpCloseButton.setName("helpCloseButton"); // NOI18N

        javax.swing.GroupLayout helpDialogeLayout = new javax.swing.GroupLayout(helpDialoge.getContentPane());
        helpDialoge.getContentPane().setLayout(helpDialogeLayout);
        helpDialogeLayout.setHorizontalGroup(
            helpDialogeLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, helpDialogeLayout.createSequentialGroup()
                .addContainerGap(479, Short.MAX_VALUE)
                .addComponent(helpCloseButton)
                .addGap(40, 40, 40))
            .addGroup(helpDialogeLayout.createSequentialGroup()
                .addGap(19, 19, 19)
                .addComponent(jLabel1, javax.swing.GroupLayout.PREFERRED_SIZE, 511, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(48, Short.MAX_VALUE))
        );
        helpDialogeLayout.setVerticalGroup(
            helpDialogeLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, helpDialogeLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel1, javax.swing.GroupLayout.PREFERRED_SIZE, 331, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 28, Short.MAX_VALUE)
                .addComponent(helpCloseButton)
                .addContainerGap())
        );

        setComponent(mainPanel);
        setMenuBar(menuBar);
        setStatusBar(statusPanel);
    }// </editor-fold>//GEN-END:initComponents

    private void gameMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_gameMenuItemActionPerformed
        JFileChooser chooser = new JFileChooser();
        PADUAFilter filter = new PADUAFilter(true);

        chooser.setFileFilter(filter);
        int returnVal = chooser.showOpenDialog(this.getFrame());
        if(returnVal == JFileChooser.APPROVE_OPTION) {
            this.generalInfoTextArea.setText("you chose to open the following game dictionary file : " +
                    chooser.getSelectedFile().getName()+"\n");
            File file = chooser.getSelectedFile();
            this.gameDictionary = new GameDictionary(file);
            int translateReturn = this.gameDictionary.translate();
            switch(translateReturn){
                case -2:
                    this.generalInfoTextArea.append("System failed to read the game dictionary file you choose\n");
                    break;
                case -3:
                    this.generalInfoTextArea.append("System failed to process the game dictionary file you choose\n");
                    break;
                case 0:
                    this.generalInfoTextArea.append("the file you choose is in the right format\n");
                    this.generalInfoTextArea.append(this.gameDictionary.output());
                    this.playGameMenu.setEnabled(true);
                    break;
            }
        }
    }//GEN-LAST:event_gameMenuItemActionPerformed

    private void proMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_proMenuItemActionPerformed
        // TODO add your handling code here:
        JFileChooser chooser = new JFileChooser();
        PADUAFilter filter = new PADUAFilter(false);
        chooser.setFileFilter(filter);
        int returnVal = chooser.showOpenDialog(this.getFrame());
        if(returnVal == JFileChooser.APPROVE_OPTION) {
            if(this.playersInfoTextArea.getText().equalsIgnoreCase("YOU HAVE NOT LOADED ANY DATASETS YET!!"))
                this.playersInfoTextArea.setText("");
            this.playersInfoTextArea.append("Proponent Dataset File: " +
                    chooser.getSelectedFile().getPath()+"\n");
            this.proFile = chooser.getSelectedFile().getPath();
        }
    }//GEN-LAST:event_proMenuItemActionPerformed

    private void oppMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_oppMenuItemActionPerformed
        // TODO add your handling code here:
        JFileChooser chooser = new JFileChooser();
        PADUAFilter filter = new PADUAFilter(false);
        chooser.setFileFilter(filter);
        int returnVal = chooser.showOpenDialog(this.getFrame());
        if(returnVal == JFileChooser.APPROVE_OPTION) {
            if(this.playersInfoTextArea.getText().equalsIgnoreCase("YOU HAVE NOT LOADED ANY DATASETS YET!!"))
                this.playersInfoTextArea.setText("");
            this.playersInfoTextArea.append("Opponent Dataset File: " +
                    chooser.getSelectedFile().getPath()+"\n");
            this.oppFile = chooser.getSelectedFile().getPath();
        }
    }//GEN-LAST:event_oppMenuItemActionPerformed

    private void loadCaseMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_loadCaseMenuItemActionPerformed
        // TODO add your handling code here:
        this.customDialog = new DynamicCaseDialog(this.getFrame(),this.gameDictionary);
        this.customDialog.pack();
        this.customDialog.setLocationRelativeTo(this.getFrame());
        this.customDialog.setVisible(true);
    }//GEN-LAST:event_loadCaseMenuItemActionPerformed

    private void proStrategyDoneButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_proStrategyDoneButtonActionPerformed

        if((!Checker.intValue(this.proConsideredLevelsTextField.getText()))||
                (!Checker.intValue(this.proMaxConfTextField.getText()))||
                (!Checker.intValue(this.proMaxLevelsTextField.getText()))||
                (!Checker.intValue(this.proMiinConfTextField.getText()))||
                (!Checker.intValue(this.proUpThresholdTextField.getText()))||
                (!Checker.intValue(this.proDistThresholdTextField.getText()))
                ){
                    JOptionPane.showMessageDialog(this.getFrame(),
                    "Please Insert VALID INTEGER INTEGER VALUES in all the text field.",
                    "Inane error",JOptionPane.ERROR_MESSAGE);
        }
        else{
            this.proStrategyDialog.hide();
            double minConf = Double.valueOf(this.proMiinConfTextField.getText());
            double maxConf = Double.valueOf(this.proMaxConfTextField.getText());
            double dTrheshold = Double.valueOf(this.proDistThresholdTextField.getText());
            double uTrheshold = Double.valueOf(this.proUpThresholdTextField.getText());
            int maxLevels = Integer.valueOf(this.proMaxLevelsTextField.getText());
            int conLevels = Integer.valueOf(this.proConsideredLevelsTextField.getText());
            int profile;
            if(this.proAgentProfileComboBox.getSelectedIndex()==0)
                profile = StrategyConstants.AGENT_PROFILE_AGREEABLE;
            else
                profile = StrategyConstants.AGENT_PROFILE_DISAGREEABLE;
            int game;
            if(this.proGameModeComboBox.getSelectedIndex()==0)
                game = StrategyConstants.GAME_MODE_GAME;
            else
                game = StrategyConstants.GAME_MODE_WIN;
            int strategyMode;
            if(this.proStrategyModeComboBox.getSelectedIndex()==0)
                strategyMode = StrategyConstants.STRATEGY_MODE_BUILD;
            else
                strategyMode = StrategyConstants.STRATEGY_MODE_DESTROY;

            this.proStrategy = new Strategy(game, profile, strategyMode);
            this.proStrategy.setConsideredLevels(conLevels);
            this.proStrategy.setMaxLevel(maxLevels);
            this.proStrategy.setMaxConf(maxConf);
            this.proStrategy.setMinConf(minConf);
            this.proStrategy.setDistThreshold(dTrheshold);
            this.proStrategy.setUpThreshold(uTrheshold);
        }
}//GEN-LAST:event_proStrategyDoneButtonActionPerformed

    private void oppStrategyOptionsDonejButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_oppStrategyOptionsDonejButtonActionPerformed
        // TODO add your handling code here:
        if((!Checker.intValue(this.oppConsideredLevelsTextField.getText()))||
                (!Checker.intValue(this.oppMaxConfTextField.getText()))||
                (!Checker.intValue(this.oppMaxLevelsTextField.getText()))||
                (!Checker.intValue(this.oppMinConfTextField.getText()))||
                (!Checker.intValue(this.oppUpThresholdTextField.getText()))||
                (!Checker.intValue(this.oppDistThresholdTextField.getText()))
                ){
                    JOptionPane.showMessageDialog(this.getFrame(),
                    "Please Insert VALID INTEGER INTEGER VALUES in all the text field.",
                    "Inane error",JOptionPane.ERROR_MESSAGE);
        }
        else{
            this.oppStrategyDialog.hide();
            double minConf = Double.valueOf(this.oppMinConfTextField.getText());
            double maxConf = Double.valueOf(this.oppMaxConfTextField.getText());
            double dTrheshold = Double.valueOf(this.oppDistThresholdTextField.getText());
            double uTrheshold = Double.valueOf(this.oppUpThresholdTextField.getText());
            int maxLevels = Integer.valueOf(this.oppMaxLevelsTextField.getText());
            int conLevels = Integer.valueOf(this.oppConsideredLevelsTextField.getText());
            int profile;
            if(this.oppAgentProfileComboBox.getSelectedIndex()==0)
                profile = StrategyConstants.AGENT_PROFILE_AGREEABLE;
            else
                profile = StrategyConstants.AGENT_PROFILE_DISAGREEABLE;
            int game;
            if(this.oppGameModeComboBox.getSelectedIndex()==0)
                game = StrategyConstants.GAME_MODE_GAME;
            else
                game = StrategyConstants.GAME_MODE_WIN;
            int strategyMode;
            if(this.oppStrategyModeComboBox.getSelectedIndex()==0)
                strategyMode = StrategyConstants.STRATEGY_MODE_BUILD;
            else
                strategyMode = StrategyConstants.STRATEGY_MODE_DESTROY;

            this.oppStrategy = new Strategy(game, profile, strategyMode);
            this.oppStrategy.setConsideredLevels(conLevels);
            this.oppStrategy.setMaxLevel(maxLevels);
            this.oppStrategy.setMaxConf(maxConf);
            this.oppStrategy.setMinConf(minConf);
            this.oppStrategy.setDistThreshold(dTrheshold);
            this.oppStrategy.setUpThreshold(uTrheshold);
        }
}//GEN-LAST:event_oppStrategyOptionsDonejButtonActionPerformed

    private void posGameMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_posGameMenuItemActionPerformed
        // TODO add your handling code here:
        this.setCurrentCase();
        short[] classifications = this.gameDictionary.getClassesAttributes();
        double[] support = {0.01, 1};
        this.dialogueGame = new DialogueGame(classifications[1], classifications[0], support);
        this.dialogueGame.setStrategy(this.proStrategy, this.oppStrategy);
        this.dialogueGame.getData(proFile, oppFile);
        this.dialogueGame.playGame(currentCase);
        this.dialogueGameTextArea.setText(this.dialogueGame.getGameHistory().output(this.gameDictionary,true));
    }//GEN-LAST:event_posGameMenuItemActionPerformed

    private void negGameMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_negGameMenuItemActionPerformed
        // TODO add your handling code here:
        this.setCurrentCase();
        short[] classifications = this.gameDictionary.getClassesAttributes();
        double[] support = {0.01, 1};
        this.dialogueGame = new DialogueGame(classifications[0], classifications[1], support);
        this.dialogueGame.setStrategy(this.proStrategy, this.oppStrategy);
        this.dialogueGame.getData(proFile, oppFile);
        this.dialogueGame.playGame(currentCase);
        this.dialogueGameTextArea.setText(this.dialogueGame.getGameHistory().output(this.gameDictionary,false));
    }//GEN-LAST:event_negGameMenuItemActionPerformed

    private void proStrategyMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_proStrategyMenuItemActionPerformed
        // TODO add your handling code here:
        this.proStrategyDialog.setSize(441, 249);
        this.proStrategyDialog.show();
    }//GEN-LAST:event_proStrategyMenuItemActionPerformed

    private void oppStrategyMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_oppStrategyMenuItemActionPerformed
        // TODO add your handling code here:
        this.oppStrategyDialog.setSize(441, 249);
        this.oppStrategyDialog.show();
    }//GEN-LAST:event_oppStrategyMenuItemActionPerformed

    private void aboutMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_aboutMenuItemActionPerformed
        // TODO add your handling code here:
        this.helpDialoge.show();
    }//GEN-LAST:event_aboutMenuItemActionPerformed

    private void mainTabbedPaneStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_mainTabbedPaneStateChanged
        // TODO add your handling code here:
        int index = mainTabbedPane.getSelectedIndex();
        Color c = new Color(229,229,244);
        mainTabbedPane.setBackgroundAt(index, c); 
    }//GEN-LAST:event_mainTabbedPaneStateChanged

    private short[] translateCase(String[] input){
        /*short[] currentCase = new short[input.length];
        for(int i=0; i<input.length; i++){
            this.currentCase[i] = this.gameDictionary.link(input[i]);
        }*/
        short[] newCase = this.gameDictionary.linkCase(input);
        return newCase;
    }
    private void outputCase(String[] stringCase){
        short[] newCase= this.translateCase(stringCase);
        this.dialogueGameTextArea.append("you have selected the following Case :\n");
        this.dialogueGameTextArea.append("{ ");
        for(int i=0; i<newCase.length; i++){
            this.dialogueGameTextArea.append(stringCase[i] + "  ");
        }
        this.dialogueGameTextArea.append("}\n ");
        this.dialogueGameTextArea.append("{ ");
        for(int i=0; i<currentCase.length; i++){
            this.dialogueGameTextArea.append(currentCase[i] + "  ");
        }
        this.dialogueGameTextArea.append("}\n ");
    }
    
    private void setCurrentCase(){
     if(this.customDialog==null){
            JOptionPane.showMessageDialog(this.getFrame(),
                    "Please insert a CASE or a CASES FILE!!!.",
                    "Inane error",JOptionPane.ERROR_MESSAGE);
        }
        else{
            String[] customDialogCase = this.customDialog.getCurrentCase();
            if(customDialogCase==null){
                JOptionPane.showMessageDialog(this.getFrame(),
                    "CURRENT CASE IS NULL!!! Please inset a case.",
                    "Inane error",JOptionPane.ERROR_MESSAGE);
            }
            else{//the actual set case code
                this.currentCase = this.gameDictionary.linkCase(customDialogCase);
            }
        }
    }
   
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel agentProfilejLabel;
    private javax.swing.JLabel agentProfilejLabel1;
    private javax.swing.JLabel armOptionsjLabel;
    private javax.swing.JLabel armOptionsjLabel1;
    private javax.swing.JLabel consideredLevelsjLabel;
    private javax.swing.JLabel consideredLevelsjLabel1;
    private javax.swing.JScrollPane dialogueGameScrollPane;
    private javax.swing.JTextArea dialogueGameTextArea;
    private javax.swing.JLabel distThresholdjLabel;
    private javax.swing.JLabel distThresholdjLabel1;
    private javax.swing.JMenuItem gameMenuItem;
    private javax.swing.JLabel gameModejLabel;
    private javax.swing.JLabel gameModejLabel1;
    private javax.swing.JScrollPane generalInfoScrollPane;
    private javax.swing.JTextArea generalInfoTextArea;
    private javax.swing.JButton helpCloseButton;
    private javax.swing.JDialog helpDialoge;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JSeparator jSeparator3;
    private javax.swing.JSeparator jSeparator5;
    private javax.swing.JMenuItem loadCaseMenuItem;
    private javax.swing.JPanel mainPanel;
    private javax.swing.JTabbedPane mainTabbedPane;
    private javax.swing.JLabel maxConfjLabel;
    private javax.swing.JLabel maxConfjLabel1;
    private javax.swing.JLabel maxLeveljLabel;
    private javax.swing.JLabel maxLeveljLabel1;
    private javax.swing.JMenuBar menuBar;
    private javax.swing.JLabel minConfjLabel;
    private javax.swing.JLabel minConfjLabel1;
    private javax.swing.JMenuItem negGameMenuItem;
    private javax.swing.JComboBox oppAgentProfileComboBox;
    private javax.swing.JTextField oppConsideredLevelsTextField;
    private javax.swing.JTextField oppDistThresholdTextField;
    private javax.swing.JComboBox oppGameModeComboBox;
    private javax.swing.JTextField oppMaxConfTextField;
    private javax.swing.JTextField oppMaxLevelsTextField;
    private javax.swing.JMenuItem oppMenuItem;
    private javax.swing.JTextField oppMinConfTextField;
    private javax.swing.JDialog oppStrategyDialog;
    private javax.swing.JMenuItem oppStrategyMenuItem;
    private javax.swing.JComboBox oppStrategyModeComboBox;
    private javax.swing.JButton oppStrategyOptionsDonejButton;
    private javax.swing.JTextField oppUpThresholdTextField;
    private javax.swing.JMenu playGameMenu;
    private javax.swing.JMenu playMenu;
    private javax.swing.JTextArea playersInfoTextArea;
    private javax.swing.JMenu playersMenu;
    private javax.swing.JMenuItem posGameMenuItem;
    private javax.swing.JComboBox proAgentProfileComboBox;
    private javax.swing.JTextField proConsideredLevelsTextField;
    private javax.swing.JTextField proDistThresholdTextField;
    private javax.swing.JComboBox proGameModeComboBox;
    private javax.swing.JTextField proMaxConfTextField;
    private javax.swing.JTextField proMaxLevelsTextField;
    private javax.swing.JMenuItem proMenuItem;
    private javax.swing.JTextField proMiinConfTextField;
    private javax.swing.JDialog proStrategyDialog;
    private javax.swing.JButton proStrategyDoneButton;
    private javax.swing.JMenuItem proStrategyMenuItem;
    private javax.swing.JComboBox proStrategyModeComboBox;
    private javax.swing.JTextField proUpThresholdTextField;
    private javax.swing.JProgressBar progressBar;
    private javax.swing.JLabel statusAnimationLabel;
    private javax.swing.JLabel statusMessageLabel;
    private javax.swing.JPanel statusPanel;
    private javax.swing.JMenu strategyMenu;
    private javax.swing.JLabel strategyModeLabel;
    private javax.swing.JLabel strategyModeLabel1;
    private javax.swing.JLabel upThresholdjLabel;
    private javax.swing.JLabel upThresholdjLabel1;
    // End of variables declaration//GEN-END:variables

    private final Timer messageTimer;
    private final Timer busyIconTimer;
    private final Icon idleIcon;
    private final Icon[] busyIcons = new Icon[15];
    private int busyIconIndex = 0;

    private JDialog aboutBox;
}
