/* Player.java
 * represents the players in PADUA game:
 * the goal of the player
 * the strategy of the player
 * and play function
 */

package paduainterface;
import java.util.*;


/*this class implements the argumentative agents in PADUA game*/
public class  Player {
    /* ------ FIELDS ------ */      
   protected short goalClass; //the class that this agent tries to prove true
   protected short counterGoalClass;//the class that this agent tries to prove false
   protected short[] currentCase;//the current case under argumentation

   public Strategy strategy; //the strategy parameters applied by this player
   //protected RuleList proposeRulesList=null;  

   protected PartialSupportTree pTree; //the p-tree representing the player's data
   protected double min_support; //The minimum support threshold of the game


  /**constructs a new object of this class*/
   public Player(short goal, short counterGoal, double support) {	
	goalClass=goal;
	counterGoalClass = counterGoal;
	min_support=support;
    }
   /**setters&getters*/
   protected Strategy getStrategy (){return this.strategy;}
   protected short getGoalClass (){return this.goalClass;}
   protected void setStrategy (Strategy newStrategy){this.strategy = newStrategy;}
   protected void setGoalClass (short goal){this.goalClass = goal;}
   protected void setCounterGoalClass (short counterGoal){this.counterGoalClass = counterGoal;}
   protected void setMinSupport(double support){this.min_support = support;}

  
   /** general methods - build T&P-Trees */
   protected void getCase(short[] inputCase) {
	currentCase = new short[inputCase.length];
	for(int i=0; i<inputCase.length; i++)
		currentCase[i] = inputCase[i];}
   /**create the P-Tree that represents the data stored in the dataFile*/
   protected void getData(String dataFile) {
	String[] args = {"-F"+dataFile, "-S"+min_support};
	pTree = new PartialSupportTree(args);
	pTree.inputDataSet();
	pTree.createPtree();
    }
   /**create the T-Tree of the instance P-Tree*/
   protected int buildTTree() {
	int maxLevel =  pTree.createTotalSupportTreeINT();
	this.strategy.setMaxLevel(maxLevel);
	return maxLevel;
    }
/*-------------------------------------------------------------------------------*/

   /* PADUA Game methods */
   /**plays the first move in the game*/
   protected Move playFirstMove(ActiveHistory gameActiveHistory){       
       return playPropose(strategy.getUpThreshold(),gameActiveHistory);
    }
   protected Move playPropose(double confidence, ActiveHistory gameActiveHistory){   	
	  RuleList ruleList = this.ProposeRule(confidence,
					       this.goalClass, gameActiveHistory);
	  if ((ruleList==null)||(ruleList.getNumCRs() ==0)){return null;}
	  else
	  {
	    this.strategy.setDefendingStatus(true);
	    /*choose best move in the list according to strategy*/
        //this.orderStrategy(ruleList);
	    RuleNode rule = ruleList.CutFirstRule();	     
	    Move move = new Move(MoveType.PROPOSE_RULE, rule);
	    return move;
	  }	
      //}
    }
   protected Move playCounterRule(Move lastMove, ActiveHistory gameActiveHistory){
	  RuleNode lastMoveRule = lastMove.getContent();
	  double confidence= lastMoveRule.getConfidence();
	  RuleList ruleList = this.ProposeRule(confidence, this.goalClass, gameActiveHistory);
	  if ((ruleList==null)||(ruleList.getNumCRs() ==0)){
	    //System.out.println("Counter Rule fails --> No rules generated!");
	    return null;
	  }
	  else
	  {
	    this.strategy.setDefendingStatus(true);
	    /*choose best move in the list according to strategy*/
	    RuleNode rule = ruleList.CutFirstRule();	     
	    Move move = new Move(MoveType.COUNTER_RULE, rule);
	    return move;
	  }	
    }
   protected Move playUnwnatedConsequences(Move lastMove){
       RuleNode lastMoveRule = lastMove.getContent();
       boolean wanted = UnwantedConsequences(lastMoveRule);
       if (!wanted){
	  Move move = new Move(MoveType.UNWANTED_CONSEQUENCES, lastMoveRule);
	  return move;	  
       }
       else{
	  //System.out.println("Unwanted Consequences fails");
	  return null;
	}	
    }
   protected Move playDistinguish(Move lastMove, ActiveHistory gameActiveHistory){
	RuleNode lastMoveRule = lastMove.getContent();
	RuleList ruleList = DistinguishDepthFirst(lastMoveRule, gameActiveHistory);
        if ((ruleList==null)||(ruleList.getNumCRs() ==0)){
	  //System.out.println("Distinguish fails");
	  return null;
	}
	else{
	  /*choose first move in the list -- to be replaced with content strategy*/
	  RuleNode rule = ruleList.GetLastRule();
	  Move move = new Move(MoveType.DISTINGUISH, rule);
	  return move;
	}
    }
   protected Move playIncreaseConfidence(Move lastMove, ActiveHistory gameActiveHistory){
	RuleNode lastMoveRule = lastMove.getContent();
	RuleList ruleList = IncreaseConfidenceDepthFirst(lastMoveRule, gameActiveHistory);	
        if ((ruleList==null)||(ruleList.getNumCRs() ==0)){
	  //System.out.println("Increase Confidence fails");
	  return null;
	}
	else{
	  RuleNode rule = ruleList.GetFirstRule();
	  Move move = new Move(MoveType.INCREASE_CONFIDENCE, rule);
	  return move;
	}		
    }
   protected Move playWithdraw(Move lastMove, ActiveHistory gameActiveHistory){
	RuleNode lastMoveRule = lastMove.getContent();
	RuleList ruleList = WithdrawUnwantedConsequences(lastMoveRule,  
			      strategy.getDefendingStatus(),gameActiveHistory);
        if ((ruleList==null)||(ruleList.getNumCRs() ==0)){
	  //System.out.println("Withdraw Unwanted Consequences fails");
	  return null;
	}
	else{
	  /*choose first move in the list -- to be replaced with content strategy*/
	  RuleNode rule = ruleList.GetFirstRule();
	  Move move = new Move(MoveType.WITHDRAW_UNWANTED_CONSEQUENCES, rule);
	  return move;
	}		
    }

   /**returns true if the rule has some unwanted consequences*/
   private boolean UnwantedConsequences(RuleNode lastMoveRule){
	short [] consequents = lastMoveRule.getConsequent();
	int i=0; 
	boolean consequencesOK=true;
	while((i<consequents.length) && (consequencesOK)){
	  int j=0; 
	  boolean found =false;
	  while((j<currentCase.length) && (!found)){
	    if((currentCase[j]==consequents[i])||
	      (consequents[i]==goalClass)||
              (consequents[i]==counterGoalClass))
		found=true;
	    else
	     j++;
	  }
	  if(found)
		i++;
	  else
	   consequencesOK=false;
	}
	return consequencesOK;
    }
   /**returns the unwanted consequences*/
   private short[] DetermineUnwantedConsequences(short [] consequents){
     AssocRuleMining arm = new AssocRuleMining();
     short[] unwantedList = null;
     for (int i =0; i< consequents.length; i++){
         boolean found = false;
         int j =0;
         while ((j<currentCase.length)&&(!found)){
           if((currentCase[j]==consequents[i])||
	      (consequents[i]==goalClass)||
              (consequents[i]==counterGoalClass))
		found=true;
	    else
	     j++;
         }
	 if (!found){
 	   short[] elem = new short[1];
	   elem[0] = consequents[i];
	   unwantedList = arm.append(unwantedList,elem);
         }
     }   
     return unwantedList;
   }
   /**returns the wanted consequences*/
   private short[] DetermineWantedConsequences(short [] consequents){
     AssocRuleMining arm = new AssocRuleMining();
     short[] wantedList = null;
     for (int i =0; i< consequents.length; i++){
         boolean found = false;
         int j =0;
         while ((j<currentCase.length)&&(!found)){
           if((currentCase[j]==consequents[i])||
	      (consequents[i]==goalClass)||
              (consequents[i]==counterGoalClass))
		found=true;
	    else
	     j++;
         }
	 if (found){
 	   short[] elem = new short[1];
	   elem[0] = consequents[i];
	   wantedList = arm.append(wantedList,elem);
         }
     }   
     return wantedList;
   }
   
   /**prooses a new rule*/
   private RuleList ProposeRule(double confidence, short goal, ActiveHistory gameActiveHistory){
        RuleList ruleList = new RuleList();
	ruleList = ProposeRulelevel1(confidence, goal,gameActiveHistory);
        RuleNode firstRule = ruleList.GetFirstRule();
	
	if((firstRule==null)||(firstRule.getConfidence()<strategy.getUpThreshold())){
	  short level = 2;
          boolean done =false;
	  while ((!done)&&(level<=currentCase.length)&&(level<strategy.getMaxLevel())){
	    RuleList newList = ProposeRulelevelN(confidence, level, goal,gameActiveHistory);
	    ruleList.merge(newList);
            firstRule = ruleList.GetFirstRule();
            if((firstRule!=null)&&(firstRule.getConfidence()>=strategy.getUpThreshold()))
             done = true;
	    else
	     level++;
	  }
	}  		
	return ruleList;
    }
   /**return the first best rule attribute-->goal*/
   private RuleList ProposeRulelevel1(double confidence,short goal, 
				      ActiveHistory gameActiveHistory){
       double minConf = strategy.getMinConf();
       if(minConf<confidence)
           minConf = confidence;
       RuleList ruleList = new RuleList();
       short i = 0;
       boolean done = false;
       while ((i<currentCase.length)&&(!done)){
           short[] level1PossibleFrequentSet = new short[2];
           level1PossibleFrequentSet[0] = currentCase[i];
           level1PossibleFrequentSet[1] = goal;
             //get the support of this set
           double support = pTree.getSupportForItemSetInTtree(level1PossibleFrequentSet);
           if(support>0){
               //produce the rule List of the given set
	       short[] restList = new short[1];
	       restList[0] = level1PossibleFrequentSet[0];
	       short[] classList = new short[1];
	       classList[0] = level1PossibleFrequentSet[1];
	       ruleList = pTree.generateClassARsFromItemsetConfStrategy
			  (restList, classList, support, 
                           minConf, strategy.getMaxConf(),
                           gameActiveHistory);
               RuleNode firstRule = ruleList.GetFirstRule();
	       if((firstRule!=null)&&(firstRule.getConfidence() >confidence))
                done=true;
	       else
	        i++;
             }
             else
	      i++;	   	
	}
        return ruleList;
    }
   /**return the first best rule A-->goal : A is a level-subset of current Case*/
   private RuleList ProposeRulelevelN(double confidence, short level, short goal,
                 			ActiveHistory gameActiveHistory){
	  double minConf = strategy.getMinConf();
	  if(minConf<confidence)
	     minConf = confidence;
	  boolean done = false;
          RuleList ruleList = new RuleList();
	  //generate levelwise subsets one by one
	  short caseSize = (short) currentCase.length;
	  SubsetsGenerator generator = new SubsetsGenerator(caseSize, level);
	  while ((generator.HasNext())&&(!done)){
	    short[] indexSet = generator.Next();
	    short[] itemSubset = new short [indexSet.length+1];
	    for(int i =0; i<indexSet.length; i++){
		itemSubset[i] = currentCase[indexSet[i]];
            }
	    
            itemSubset[indexSet.length] = goal;
	    //for each generated l-subset check for support
            Arrays.sort(itemSubset);
	     double support = pTree.getSupportForItemSetInTtree(itemSubset);
	     //if support > min --> produce rule and check for confidence
             if(support>0){
	       short[] restList = new short[indexSet.length];
	       for(int i =0; i<indexSet.length; i++){
			restList[i] = currentCase[indexSet[i]];
            	}
               Arrays.sort(restList);
	       short[] classList = new short[1];
	       classList[0] = goal;
	       ruleList = pTree.generateClassARsFromItemsetConfStrategy
			  (restList, classList, support, 
                           minConf, strategy.getMaxConf(),
                           gameActiveHistory);
	       RuleNode rule =  ruleList.GetFirstRule();
	       if((rule!=null)&&(rule.getConfidence() >confidence)
                              &&(rule.getConfidence() >=strategy.getUpThreshold()))
                done=true;
	     }
	  }
        return ruleList;
    }
   
   /**delete the case attributes used in the input rule*/
   private short[] subRuleOfCase(short[] inputCase, short[] antecednt, short[]consequent){
     AssocRuleMining arm = new AssocRuleMining();
     short[] newCase = null;
     for(int i =0; i<inputCase.length; i++){
         boolean found = false;
         int j =0;
         while ((j<antecednt.length)&&(!found)){
           if(inputCase[i] == antecednt [j])
		found = true;
	   else
              j++;
         }
	 if (!found){
           j =0;
           while ((j<consequent.length)&&(!found)){
            if(inputCase[i] == consequent [j])
		found = true;
	    else
              j++;
           }
	  if (!found){
           short[] elem = new short[1];
	   elem[0] = currentCase[i];
	   newCase = arm.append(newCase,elem);
	  }
         }
     }   
     return newCase;
   }

   /*return the first best rule A+B-->C 
       which confidence is lower than the original rule (A-->C)*/
   private RuleList DistinguishDepthFirst(RuleNode inputRule, 
					ActiveHistory gameActiveHistory){
	//produce the set to be porcessed
	short[] antecednt = inputRule.getAntecednt();
	short[] consequent = inputRule.getConsequent();
	//double confidence = inputRule.getConfidence();
	double distConf = strategy.getDistThreshold();
	double minConf = strategy.getMinConf();
	//delete the case attributes used in the input rule 
	short[] newCase = subRuleOfCase(currentCase,antecednt, consequent);
        if(newCase!= null){
          RuleList ruleList = DistinguishDepthFirstlevel1
		   (strategy.getMaxLevel(), newCase, 
		    antecednt, consequent, minConf, distConf, 
		    gameActiveHistory);
          return ruleList;
        }
        else
        return null;
    }
   private RuleList DistinguishDepthFirstlevel1
	(int maxLevel,short[] newCase, 
	 short[] antecednt, short[] consequent, 
	 double minConfidence, double disConfidence,
	 ActiveHistory gameActiveHistory){

          RuleList ruleList = new RuleList();
	  short i = 0;
	  boolean done = false;
	  while ((i<newCase.length)&&(!done))
	  {
	     //expand the  antecedent with the current element of newCase
             short[] currentItemSet = new short [1];
	     currentItemSet[0] = newCase[i];
             AssocRuleMining arm = new AssocRuleMining();
	     short[] expAntecedent = arm.append(antecednt, currentItemSet);
 	     Arrays.sort(expAntecedent);
	     //merge both expanded antecednt and consequent
	     short expItemSet[] = arm.append(expAntecedent, consequent);
	     
	     Arrays.sort(expItemSet);
             //get the support of this set
	     double support;
	     if(expItemSet.length <=maxLevel)
                support = pTree.getSupportForItemSetInTtree(expItemSet);	
	     else
	        support = 0;
	     if(support>0)
             { 	
	       //check confidence of  new rule
	       double ruleConf = pTree.getConfidence(expAntecedent, support);
	       if(ruleConf< minConfidence)
	       {
		   RuleNode rule = new RuleNode (expAntecedent,
		     			   consequent,disConfidence);
		    boolean found = gameActiveHistory.search(rule);
		    if(!found)
		    {
		      ruleList.insertRuleintoRulelist(expAntecedent,
		     				   consequent,disConfidence);
                      if(ruleConf<= disConfidence)
		         done=true;
		      else
		        i++;
		    }
		  else //the rule already exists --> search subtree of this rule???
		  {
		    /*ruleList = pTree.distinguishConclusionHistory
			     (expAntecedent, consequent, 
                              minConfidence, disConfidence,gameActiveHistory);
		    RuleNode lastRule = ruleList.GetLastRule();
		    if((lastRule!=null)&&(lastRule.getConfidence() <= disConfidence)) 
                       done=true;
		    else*/ i++;
		  }
	       }
	       else //confidence
	       //check super itemsets of this node
		{
		  ruleList = pTree.distinguishConclusionHistory
			     (expAntecedent, consequent, 
                              minConfidence, disConfidence,gameActiveHistory);
		  RuleNode lastRule = ruleList.GetLastRule();
		  if((lastRule!=null)&&(lastRule.getConfidence() <= disConfidence)) 
                       done=true;
		  else i++;
		}
             }
             else //support
	      i++;	   	
	}
        return ruleList;
    }

   /** return the first best rule A+B-->C 
       which confidence is lower than the original rule (A-->C)*/
   private RuleList IncreaseConfidenceDepthFirst(RuleNode inputRule, 
	   ActiveHistory gameActiveHistory){
	//produce the set to be porcessed
	short[] antecednt = inputRule.getAntecednt();
	short[] consequent = inputRule.getConsequent();
	double confidence = inputRule.getConfidence();
	double minConf = strategy.getMinConf();
	if(confidence<minConf)
	  confidence = minConf;
	//delete the case attributes used in the input rule 
	short[] newCase = subRuleOfCase(currentCase,antecednt, consequent);
	RuleList ruleList = IncreaseConfidenceDepthFirstLevel1
		   (strategy.getMaxLevel(), newCase, 
		    antecednt, consequent, 
		    confidence, strategy.getUpThreshold(),
		    gameActiveHistory);
	return ruleList;
    }

  private RuleList IncreaseConfidenceDepthFirstLevel1
	(int maxLevel,short[] newCase, short[] antecednt, short[] consequent, 
	   double minConfidence, double upThreshold,
	   ActiveHistory gameActiveHistory){
	
        RuleList ruleList = new RuleList();
	short i = 0;
	boolean done = false;
	while ((newCase!=null)&&(i<newCase.length)&&(!done))
	{
	     //expand the  antecedent with the current element of newCase
             short[] currentItemSet = new short [1];
	     currentItemSet[0] = newCase[i];
             AssocRuleMining arm = new AssocRuleMining();
	     short[] expAntecedent = arm.append(antecednt, currentItemSet);
 	     Arrays.sort(expAntecedent);
	     //merge both expanded antecednt and consequent
	     short expItemSet[] = arm.append(expAntecedent, consequent);
	     
	     Arrays.sort(expItemSet);
             //get the support of this set
	     double support;
	     if(expItemSet.length <=maxLevel)
             support = pTree.getSupportForItemSetInTtree(expItemSet);	
	     else
	     support = 0;
	     if(support>0){
		//check confidence of  new rule
	        double confidence = pTree.getConfidence(expAntecedent, support);
            if(confidence> minConfidence)
            {
                RuleNode rule = new RuleNode (expAntecedent,
		    			   consequent,confidence);
                boolean found = gameActiveHistory.search(rule);
                if(!found)
                {
                    ruleList.insertRuleintoRulelist(expAntecedent,
		    				   consequent,confidence);
                    if(confidence>= upThreshold)    done=true;
                    else i++;
                }
                else{ //the rule already exists --> search subtree of this rule???
		    /*ruleList = pTree.increaseConfidenceConclusionHistory
			     (expAntecedent, consequent, 
			      minConfidence,upThreshold,
			      gameActiveHistory);
		  RuleNode firstRule = ruleList.GetFirstRule();
		  if((firstRule!=null)&&(firstRule.getConfidence() >=upThreshold)) 
                       done=true;
		  else*/ i++;
		  }
		}
		else // confidence
		{
		    //check super itemsets of this node
		    ruleList = pTree.increaseConfidenceConclusionHistory
			     (expAntecedent, consequent, 
			      minConfidence,upThreshold,
			      gameActiveHistory);
		  RuleNode firstRule = ruleList.GetFirstRule();
		  if((firstRule!=null)&&(firstRule.getConfidence() >=upThreshold)) 
                       done=true;
		    else i++;
		}
	     }
	     else i++; //support
	}
	return ruleList;
    }

  private RuleList WithdrawUnwantedConsequences
		   (RuleNode inputRule, boolean defending,
		    ActiveHistory gameActiveHistory){

	RuleList ruleList = new RuleList();
	//produce the set to be porcessed
	short[] antecednt = inputRule.getAntecednt();
	short[] consequent = inputRule.getConsequent();
	double confidence = inputRule.getConfidence();
	//determine the unwanted consequences
	short[] wantedConsequneces = DetermineWantedConsequences(consequent);
	AssocRuleMining arm = new AssocRuleMining();
	short[] newItemSet = arm.append(antecednt,wantedConsequneces);
	Arrays.sort(newItemSet);
	double support = pTree.getSupportForItemSetInTtree(newItemSet);
	if(support>0)
	{
	   //get confidence
	   double withConfidence = pTree.getConfidence(antecednt, support);
	   if (defending)
	   {
		if(withConfidence>=strategy.getMinConf())
	   	{		
 		  RuleNode rule = new RuleNode (antecednt,
		    			   wantedConsequneces,withConfidence);
		  boolean found = gameActiveHistory.search(rule);
		  if(!found)
		  {
		    ruleList.insertRuleintoRulelist(antecednt,
		    			   wantedConsequneces,withConfidence);
		  }
	   	}
	   }
	   else
	   {
		if(withConfidence<strategy.getMinConf())
	   	{		
		   RuleNode rule = new RuleNode (antecednt,
		    			   wantedConsequneces,withConfidence);
		    boolean found = gameActiveHistory.search(rule);
		    if(!found)
		    {
		      ruleList.insertRuleintoRulelist(antecednt,
		    			   wantedConsequneces,withConfidence);
		    }
	   	}
	   }
	   
	   /*else search withing support subtree*/
	   //short[] newCase = subRuleOfCase(currentCase,antecednt, consequent);
	}
	return ruleList;	
    }
  
  /** play FUNCTION  */
  private boolean checkAgreeGame(Move lastMove){
      AssocRuleMining arm = new AssocRuleMining();
      short[] itemset = arm.append(lastMove.getContent().getAntecednt(), lastMove.getContent().getConsequent());
      Arrays.sort(itemset);
      double support = this.pTree.getSupportForItemSetInTtree(itemset);
      if(support<this.min_support) return false;
      double conf = this.pTree.getConfidence(lastMove.getContent().getAntecednt(), support);
      if(conf>=this.strategy.getMinConf())
          return true;
      else return false;
  }
  private boolean checkAgreeWin(Move lastMove){
      if(this.checkAgreeGame(lastMove)) return true;
      AssocRuleMining arm = new AssocRuleMining();
      short[] itemset = arm.append(lastMove.getContent().getAntecednt(), lastMove.getContent().getConsequent());
      Arrays.sort(itemset);
      double support = this.pTree.getSupportForItemSetInTtree(itemset);
      if(support<this.min_support) return false;
      if(this.checkRulelevel1(lastMove.getContent()))
          return true;
      if(this.checkRulelevelN(lastMove.getContent(),(short)2))
          return true;
      return false;
  }
  protected short[] append(short[] itemSet1, short item) {
	if (itemSet1 == null) {
        short[] itemSet = new short[1];
        itemSet[0] = item;
        return itemSet;
    }
	short[] newItemSet = new short[itemSet1.length+1];
	for(int index1=0;index1<itemSet1.length;index1++) {
	    newItemSet[index1]=itemSet1[index1];
    }
    newItemSet[itemSet1.length] = item;
	return(newItemSet);
  }
  private boolean checkRulelevel1(RuleNode rule){
       double minConf = rule.getConfidence();
       short[] consequents = rule.getConsequent();
       short[] newCase = this.subRuleOfCase(this.currentCase, rule.getAntecednt(), rule.getConsequent());
       if(newCase==null) return false;
       if(newCase.length==0) return false;
       for(int i=0; i<newCase.length; i++){
           short[] level1PossibleFrequentSet = this.append(consequents, newCase[i]);
           Arrays.sort(level1PossibleFrequentSet);
           double support = this.pTree.getSupportForItemSetInTtree(level1PossibleFrequentSet);
           if(support<this.min_support) return false;
           short[] newAnt = new short[1];
           newAnt[0] = newCase[i];
           if(this.pTree.getConfidence(newAnt, support)>=minConf)
               return true;
           
       }
       return false;
    }
   /**return the first best rule A-->goal : A is a level-subset of current Case*/
  private boolean checkRulelevelN(RuleNode rule, short level){
      if(level>this.strategy.getMaxLevel()) return false;
	  double minConf = rule.getConfidence();
      short[] newCase = this.subRuleOfCase(this.currentCase, rule.getAntecednt(), rule.getConsequent());
      if(newCase==null) return false;
      if(newCase.length==0) return false;
	  SubsetsGenerator generator = new SubsetsGenerator((short)newCase.length, level);
      while (generator.HasNext()){
          short[] indexSet = generator.Next();
          short[] newCaseSubset = new short [indexSet.length];
          for(int i =0; i<indexSet.length; i++){
              newCaseSubset[i] = newCase[indexSet[i]];
          }
          AssocRuleMining arm = new AssocRuleMining();
          short[] itemSubset = arm.append(newCaseSubset, rule.getConsequent());
          Arrays.sort(itemSubset);
          double support = pTree.getSupportForItemSetInTtree(itemSubset);
          if(support<this.min_support) return false;
          if(this.pTree.getConfidence(itemSubset,support)>=minConf)
              return true;
      }
      short newLevel = (short) (level+1);
      return checkRulelevelN(rule, newLevel);
    }

  protected Move play(Move lastMove, ActiveHistory gameActiveHistory){
      boolean agree = false;
      if(this.strategy.getAgentProfile()==StrategyConstants.AGENT_PROFILE_AGREEABLE){
          if(this.strategy.getGameMode()==StrategyConstants.GAME_MODE_WIN){
              agree = this.checkAgreeWin(lastMove);
          }
          else {
              agree = this.checkAgreeGame(lastMove);
          }
      }
      if(agree) return null;
      if(this.strategy.getStrategyMode()==StrategyConstants.STRATEGY_MODE_BUILD)
          return this.playBuild(lastMove,gameActiveHistory);
      else
          return this.playDestroy(lastMove,gameActiveHistory);
  }
  private Move playDestroy(Move lastMove, ActiveHistory gameActiveHistory){
      //simplest plan-- to be replaced with overall strategy
      Move move = null;
      double lastMoveConf = lastMove.getContent().getConfidence();
      if (strategy.getMinConf()<lastMoveConf)
          strategy.setMinConf(lastMoveConf);
      switch (lastMove.getType()){
         case MoveType.PROPOSE_RULE://propose rule
             strategy.setDefendingStatus(false);
             move = this.playUnwnatedConsequences(lastMove);
             if (move == null){
                 move = this.playDistinguish(lastMove,gameActiveHistory);
                 if (move == null){
                     move = this.playCounterRule(lastMove,gameActiveHistory);
                     if (move != null)// counter rule ok
                         gameActiveHistory.insertRuleintoActiveHistory(this.goalClass, move);
                 }
                 else //distinguish ok
                     gameActiveHistory.insertRuleintoActiveHistory(this.goalClass, move);
             }
             else //unwanted ok
                 gameActiveHistory.insertRuleintoActiveHistory(goalClass, move);
	 break;
	/**********************************************************************/
	case MoveType.DISTINGUISH://distinguish
	 move = this.playUnwnatedConsequences(lastMove);
	 if (move == null){
           move = this.playIncreaseConfidence(lastMove,gameActiveHistory);
           if (move == null){
               if(strategy.getDefendingStatus()){
                   double confidence = lastMove.getContent().getConfidence();
                   move = this.playPropose(confidence, gameActiveHistory);
               }
               else
                   move = this.playCounterRule(lastMove,gameActiveHistory);
               if (move != null)// counter rule ok
                   gameActiveHistory.insertRuleintoActiveHistory(this.goalClass, move);
           }
           else //playIncreaseConfidence ok
               gameActiveHistory.insertRuleintoActiveHistory(this.goalClass, move);
     }
	 else //unwanted ok
           gameActiveHistory.insertRuleintoActiveHistory(this.goalClass, move);
	 break;
	/**********************************************************************/
	case MoveType.UNWANTED_CONSEQUENCES://unwanted consequences
	 move = this.playWithdraw(lastMove,gameActiveHistory);
     if (move == null){
         double confidence = lastMove.getContent().getConfidence();
         move = this.playPropose(confidence, gameActiveHistory);
         if (move != null)//propose ok
               gameActiveHistory.insertRuleintoActiveHistory(this.goalClass, move);
     }
	 else //withdraw ok
           gameActiveHistory.insertRuleintoActiveHistory(this.goalClass, move);
	 break;
	/**********************************************************************/
	case MoveType.COUNTER_RULE://counter rule
	 strategy.setDefendingStatus(false);
	 move = this.playUnwnatedConsequences(lastMove);
	 if (move == null){
           move = this.playDistinguish(lastMove,gameActiveHistory); 
	   if (move == null){
             double confidence = lastMove.getContent().getConfidence();
             move = this.playPropose(confidence, gameActiveHistory);
             if (move != null)// propose ok
                 gameActiveHistory.insertRuleintoActiveHistory(this.goalClass, move);
	   }
	   else //distinguish ok
             gameActiveHistory.insertRuleintoActiveHistory(this.goalClass, move);
	 }
	 else //unwanted ok
           gameActiveHistory.insertRuleintoActiveHistory(this.goalClass, move);
	 break;
	/**********************************************************************/
  	case MoveType.INCREASE_CONFIDENCE://increase confidence
	 move = this.playUnwnatedConsequences(lastMove);
	 if (move == null){
           move = this.playDistinguish(lastMove, gameActiveHistory); 
	   if (move == null){
  	     move = this.playCounterRule(lastMove, gameActiveHistory);	
             if (move != null)// counter rule ok
                 gameActiveHistory.insertRuleintoActiveHistory(this.goalClass, move);
	   }
	   else //distinguish ok
             gameActiveHistory.insertRuleintoActiveHistory(this.goalClass, move);
	 }
	 else //unwanted ok
           gameActiveHistory.insertRuleintoActiveHistory(this.goalClass, move);
	 break;
	/**********************************************************************/
  	case MoveType.WITHDRAW_UNWANTED_CONSEQUENCES://withdraw unwanted consequences
	 move = this.playUnwnatedConsequences(lastMove);
	 if (move == null){
           move = this.playDistinguish(lastMove, gameActiveHistory); 
	   if (move == null){
  	     move = this.playCounterRule(lastMove,gameActiveHistory);	
             if (move != null)// counter rule ok
                 gameActiveHistory.insertRuleintoActiveHistory(this.goalClass, move);
	   }
	   else //distinguish ok
             gameActiveHistory.insertRuleintoActiveHistory(this.goalClass, move);
	 }
	 else //unwanted ok
           gameActiveHistory.insertRuleintoActiveHistory(this.goalClass, move);
	 break;
	/**********************************************************************/
      }
      return move;	
    }
  private Move playBuild(Move lastMove, ActiveHistory gameActiveHistory){
      //simplest plan-- to be replaced with overall strategy
      Move move = null;
      double lastMoveConf = lastMove.getContent().getConfidence();
      if (strategy.getMinConf()<lastMoveConf) strategy.setMinConf(lastMoveConf);
      switch (lastMove.getType()){
         case MoveType.PROPOSE_RULE://propose rule
             strategy.setDefendingStatus(false);
             move = this.playUnwnatedConsequences(lastMove);
             if (move == null){
                 move = this.playCounterRule(lastMove,gameActiveHistory);
                 if (move == null){
                     move = this.playDistinguish(lastMove,gameActiveHistory);
                     if (move != null)// distinguish
                         gameActiveHistory.insertRuleintoActiveHistory(this.goalClass, move);
                 }
                 else //unwanted ok
                     gameActiveHistory.insertRuleintoActiveHistory(this.goalClass, move);
             }
             else //unwanted ok
                 gameActiveHistory.insertRuleintoActiveHistory(goalClass, move);
	 break;
	/**********************************************************************/
	case MoveType.DISTINGUISH://distinguish
	 move = this.playUnwnatedConsequences(lastMove);
	 if (move == null){
         if(strategy.getDefendingStatus()){
             double confidence = lastMove.getContent().getConfidence();
             move = this.playPropose(confidence, gameActiveHistory);
         }
         else   move = this.playCounterRule(lastMove,gameActiveHistory);
         if (move == null){
             move = this.playIncreaseConfidence(lastMove,gameActiveHistory);
             if (move != null)// increaese confidence ok
                 gameActiveHistory.insertRuleintoActiveHistory(this.goalClass, move);
         }
         else //counter / propose ok
             gameActiveHistory.insertRuleintoActiveHistory(this.goalClass, move);
     }
	 else //unwanted ok
           gameActiveHistory.insertRuleintoActiveHistory(this.goalClass, move);
	 break;
	/**********************************************************************/
	case MoveType.UNWANTED_CONSEQUENCES://unwanted consequences
	 move = this.playWithdraw(lastMove,gameActiveHistory);
     if (move == null){
         double confidence = lastMove.getContent().getConfidence();
         move = this.playPropose(confidence, gameActiveHistory);
         if (move != null)//propose ok
             gameActiveHistory.insertRuleintoActiveHistory(this.goalClass, move);
	 }
	 else //withdraw ok
           gameActiveHistory.insertRuleintoActiveHistory(this.goalClass, move);
	 break;
	/**********************************************************************/
	case MoveType.COUNTER_RULE://counter rule
	 strategy.setDefendingStatus(false);
	 move = this.playUnwnatedConsequences(lastMove);
	 if (move == null){
         double confidence = lastMove.getContent().getConfidence();
         move = this.playPropose(confidence, gameActiveHistory);
           if (move == null){
               move = this.playDistinguish(lastMove,gameActiveHistory);
               if (move != null)// distinguish ok
                   gameActiveHistory.insertRuleintoActiveHistory(this.goalClass, move);
           }
           else //propose ok
             gameActiveHistory.insertRuleintoActiveHistory(this.goalClass, move);
     }
	 else //unwanted ok
           gameActiveHistory.insertRuleintoActiveHistory(this.goalClass, move);
	 break;
	/**********************************************************************/
  	case MoveType.INCREASE_CONFIDENCE://increase confidence
	 move = this.playUnwnatedConsequences(lastMove);
	 if (move == null){
         move = this.playCounterRule(lastMove, gameActiveHistory);
         if (move == null){
             move = this.playDistinguish(lastMove, gameActiveHistory);
             if (move != null)// distinguish ok
                 gameActiveHistory.insertRuleintoActiveHistory(this.goalClass, move);
         }
         else //counter rule ok
             gameActiveHistory.insertRuleintoActiveHistory(this.goalClass, move);
     }
	 else //unwanted ok
           gameActiveHistory.insertRuleintoActiveHistory(this.goalClass, move);
	 break;
	/**********************************************************************/
  	case MoveType.WITHDRAW_UNWANTED_CONSEQUENCES://withdraw unwanted consequences
	 move = this.playUnwnatedConsequences(lastMove);
	 if (move == null){
         move = this.playCounterRule(lastMove,gameActiveHistory);
         if (move == null){
             move = this.playDistinguish(lastMove, gameActiveHistory);
             if (move != null)// distinguish ok
                 gameActiveHistory.insertRuleintoActiveHistory(this.goalClass, move);
         }
         else //counter rule ok
             gameActiveHistory.insertRuleintoActiveHistory(this.goalClass, move);
     }
	 else //unwanted ok
           gameActiveHistory.insertRuleintoActiveHistory(this.goalClass, move);
	 break;
	/**********************************************************************/
      }
      return move;
    }

}//end of the class
