package paduainterface;
/* Class structure

AssocRuleMining
      |
      +-- RuleList
	    |
      	    +-- RuleNode
 * represents the association rules played by the players
 */

// Java packages
import java.util.*;

public class RuleNode {
    /** Antecedent of AR. */
    protected short[] antecedent;
    /** Consequent of AR. */
    protected short[] consequent;
    
    /** The confidence value associate with the rule represented by this node. */
    protected double confidenceForRule=0.0;
    /** Link to next node */
    RuleNode next = null;
    protected boolean isfirst=true;
	
    /** Three argument constructor
	@param antecedent the antecedent (LHS) of the AR.
    	@param consequent the consequent (RHS) of the AR.
    	@param support the associated confidence value. */
    public RuleNode(short[] ante, short[]cons, double confValue) {
        antecedent        = ante;
        consequent        = cons;
        confidenceForRule = confValue;
    }
    
    /**setters & getters*/
    protected short[] getAntecednt(){return this.antecedent;}	
    protected short[] getConsequent(){return this.consequent;} 
    protected double getConfidence(){return this.confidenceForRule;}
    protected void setAntecednt(short[] newAnt){
        AssocRuleMining arm = new AssocRuleMining();
        this.antecedent = arm.copyItemSet(newAnt);
    }
    protected void setConsequent(short[] newCon){
        AssocRuleMining arm = new AssocRuleMining();
        this.consequent = arm.copyItemSet(newCon);
    } 
    protected  void setConfidence(double newConf){this.confidenceForRule = newConf;}
    
    /**outputs*/
    protected void output(){
        AssocRuleMining arm = new AssocRuleMining();
        arm.outputItemSet(this.antecedent);
        System.out.print(" -> ");
        arm.outputItemSet(this.consequent);
        System.out.print("\t\t" + this.confidenceForRule);
        System.out.print("\n");
    }

    private String itemsetToString(short[] itemset) {
	// Loop through item set elements
	String itemsetString="";
	if (itemset == null) itemsetString ="null";
	else {
            int counter = 0;
	    for (int index=0;index<itemset.length;index++) {
	        if (counter == 0) {
	    	    counter++;
		    itemsetString = itemsetString +" {";
		    }
	        else itemsetString = itemsetString +" ";
	        itemsetString = itemsetString + String.valueOf(itemset[index]);
		}
	    itemsetString = itemsetString +"} ";
	    }
         return itemsetString;
	}
    protected String getString(){
        String ruleString = itemsetToString(this.antecedent); 
        ruleString = ruleString + " -> ";
        ruleString = ruleString +itemsetToString(this.consequent);
        ruleString = ruleString +"\t\t";
        ruleString = ruleString + String.valueOf(this.confidenceForRule);
        ruleString = ruleString +"\n";
        return ruleString;
    }
    protected String output(GameDictionary dictionary){
        String output = "The case has the following features : " ;
        for(int i=0; i<this.antecedent.length-1; i++){
            output+=dictionary.linkOutput(this.antecedent[i]) + " , ";
        }
        output+= dictionary.linkOutput(this.antecedent[this.antecedent.length-1]) + "\n";
        output+= "Therefore The case should calssify as : ";
        for(int i=0; i<this.consequent.length-1; i++){
            output+=dictionary.linkOutput(this.consequent[i]) + " , ";
        }
        output+= dictionary.linkOutput(this.consequent[this.consequent.length-1]) + "\n";
        output+="with confidence = " + this.confidenceForRule + "\n";
        return output;
    }
    /*EQUAL*/
    protected static boolean equals(RuleNode rule1, RuleNode rule2) {
        if(rule1.getConfidence()==rule2.getConfidence()){
            short[] ant1 = rule1.getAntecednt();
            short[] ant2 = rule2.getAntecednt();
            Arrays.sort(ant1); 
            Arrays.sort(ant2); 
            if (Arrays.equals(ant1,ant2)){
                short[] con1 = rule1.getConsequent();
                short[] con2 = rule2.getConsequent();
                Arrays.sort(con1); 
                Arrays.sort(con2);               
                if (Arrays.equals(con1,con2)) return true;
                else return false;
            }
            else return false;
        }
        else return false;
     }  
    protected boolean equals(RuleNode rule) {
        if(this.getConfidence()==rule.getConfidence()){
            short[] ant1 = this.getAntecednt();
            short[] ant2 = rule.getAntecednt();
            Arrays.sort(ant1); 
            Arrays.sort(ant2); 
            if (Arrays.equals(ant1,ant2)){
                short[] con1 = this.getConsequent();
                short[] con2 = rule.getConsequent();
                Arrays.sort(con1); 
                Arrays.sort(con2);               
                if (Arrays.equals(con1,con2)) return true;
                else return false;
            }
            else return false;
        }
        else return false;
     }  
}
