/* Strategy.java
 * contains PADUA strategy parameters
 */

package paduainterface;

public class  Strategy {
  /* ------ FIELDS ------ */
  protected int gameMode = StrategyConstants.GAME_MODE_WIN; //1 = win, 2 = game
  protected int agentProfile = StrategyConstants.AGENT_PROFILE_DISAGREEABLE; //1 = disagreeable, 2 = agreeable
  protected int strategyMode = StrategyConstants.STRATEGY_MODE_BUILD; //1 = build, 2 = destroy

  protected double min_conf = 60.0;//the minimum confidence threshold for a rule to be accepted
  protected double max_conf = 100;//the maximum confidence threshold for a rule to be accepted
  protected double dist_threshold=40.0;//maximum confidence for a distinguished rule to be accepted
  /*max confidence for a rule of a given level in a drop down mode to be accepted*/
  protected double drop_threshold = 20.0;
  /*max confidence for a rule of a given level in an upward mode to be accepted*/
  protected double up_threshold=70.0;
  protected int max_level;//maximum number of levels to be considered
  protected int considered_levels_no=5;//maximum increase in levels to be considered
  /*indicates whether the player is in defending or attacking mode*/
  protected boolean defending=true; 

  /* ------ CONSTRUCTORS ------ */
  public Strategy(){}
  public Strategy(int game, int profile, int strategy){
    this.gameMode = game;
    this.agentProfile = profile;
    this.strategyMode = strategy;
  }
  public Strategy(double minConf, double maxConf, double distThreshold,
                  double dropThreshold, double upThreshold){
    this.min_conf = minConf;
    this.max_conf = maxConf;
    this.dist_threshold=distThreshold;
    this.drop_threshold = dropThreshold;
    this.up_threshold=upThreshold;
  }

  /* ------ SETTERS&GETTERS ------ */
   protected int getGameMode (){ return this.gameMode;}
   protected void setGameMode (int game){ this.gameMode = game;}
   protected int getAgentProfile (){ return this.agentProfile;}
   protected void setAgentProfile (int profile){this.agentProfile = profile;}
   protected int getStrategyMode (){return this.strategyMode;}
   protected void setStrategyMode (int strategy){this.strategyMode = strategy;}

   protected double getMinConf (){return this.min_conf;}
   protected void setMinConf (double minConf){this.min_conf = minConf;}
   protected double getMaxConf (){return this.max_conf;}
   protected void setMaxConf (double maxConf){ this.max_conf = maxConf;}

   protected double getDistThreshold (){return this.dist_threshold;}
   protected void setDistThreshold (double distThreshold){this.dist_threshold = distThreshold;}

   protected double getDropThreshold (){ return this.drop_threshold;}
   protected void aetDropThreshold (double dropThreshold){ this.drop_threshold = dropThreshold;}

   protected double getUpThreshold (){return this.up_threshold;}
   protected void setUpThreshold (double upThreshold){this.up_threshold = upThreshold;}

   protected int getMaxLevel(){return this.max_level;}
   protected void setMaxLevel (int maxLevel){this.max_level = maxLevel;}
   protected int getConsideredLevels (){return this.considered_levels_no;}
   protected void setConsideredLevels (int levelIncrease){this.considered_levels_no = levelIncrease;}

   protected boolean getDefendingStatus (){return this.defending;}
   protected void setDefendingStatus (boolean defendingStatus){this.defending = defendingStatus;}

   protected void setTreeLevels (int maxLevel, int levelIncrease){
    this.max_level = maxLevel;
    this.considered_levels_no=levelIncrease;
   }
   
   protected void output(){
    System.out.println("min_conf = " + min_conf);
    System.out.println("max_conf = " + max_conf);
    System.out.println("dist_threshold = " + dist_threshold);
    System.out.println("drop_threshold = " + drop_threshold);
    System.out.println("up_threshold = " + up_threshold);
    System.out.println("max_level = " + max_level);
    System.out.println("considered_levels_no = " + considered_levels_no);
    System.out.println("defending = " + defending);
  }
  protected String getName(){
      String strategyName;
      if(this.agentProfile==StrategyConstants.AGENT_PROFILE_AGREEABLE)
          strategyName = "Agent Profile : Agreeable\n";
      else
          strategyName = "Agent Profile : Disgreeable\n";
      if(this.strategyMode==StrategyConstants.STRATEGY_MODE_BUILD)
          strategyName += "Strategy Mode: Build\n";
      else
          strategyName += "Strategy Mode: Destroy\n";
      if(this.gameMode==StrategyConstants.GAME_MODE_WIN)
          strategyName += "Game Mode: Win\n";
      else
          strategyName += "Game Mode: Game\n";

      return strategyName;
  }
}//end of the class
